@@ -1,14 +1,15 @@
 from collections import OrderedDict
 from collections.abc import Mapping
+from typing import Tuple
 
 
 class Serializable:
 
-    _state_dict_all_req_keys = ()  # type: tuple
-    _state_dict_one_of_opt_keys = ()  # type: tuple
+    _state_dict_all_req_keys: Tuple = ()
+    _state_dict_one_of_opt_keys: Tuple = ()
 
     def state_dict(self) -> OrderedDict:
-        pass
+        raise NotImplementedError
 
     def load_state_dict(self, state_dict: Mapping) -> None:
         if not isinstance(state_dict, Mapping):
