@@ -1,34 +1,37 @@
 import numbers
 import warnings
 from functools import partial
-from typing import Any, Callable, Dict, Iterable, Mapping, Optional, Sequence, Tuple, Union, cast
+from typing import Any, Callable, cast, Dict, Iterable, Mapping, Optional, Sequence, Union
 
 import torch
 import torch.nn as nn
-from torch.optim.lr_scheduler import _LRScheduler
 from torch.optim.optimizer import Optimizer
 from torch.utils.data.distributed import DistributedSampler
 
+# https://github.com/pytorch/ignite/issues/2773
+try:
+    from torch.optim.lr_scheduler import LRScheduler as PyTorchLRScheduler
+except ImportError:
+    from torch.optim.lr_scheduler import _LRScheduler as PyTorchLRScheduler
+
 import ignite.distributed as idist
 from ignite.contrib.handlers import (
     ClearMLLogger,
-    LRScheduler,
+    global_step_from_engine,
     MLflowLogger,
     NeptuneLogger,
     PolyaxonLogger,
     ProgressBar,
     TensorboardLogger,
-    TrainsLogger,
     VisdomLogger,
     WandBLogger,
-    global_step_from_engine,
 )
 from ignite.contrib.handlers.base_logger import BaseLogger
-from ignite.contrib.handlers.param_scheduler import ParamScheduler
 from ignite.contrib.metrics import GpuInfo
 from ignite.engine import Engine, Events
 from ignite.handlers import Checkpoint, DiskSaver, EarlyStopping, TerminateOnNan
 from ignite.handlers.checkpoint import BaseSaveHandler
+from ignite.handlers.param_scheduler import ParamScheduler
 from ignite.metrics import RunningAverage
 from ignite.utils import deprecated
 
@@ -39,7 +42,7 @@ def setup_common_training_handlers(
     to_save: Optional[Mapping] = None,
     save_every_iters: int = 1000,
     output_path: Optional[str] = None,
-    lr_scheduler: Optional[Union[ParamScheduler, _LRScheduler]] = None,
+    lr_scheduler: Optional[Union[ParamScheduler, PyTorchLRScheduler]] = None,
     with_gpu_stats: bool = False,
     output_names: Optional[Iterable[str]] = None,
     with_pbars: bool = True,
@@ -53,43 +56,43 @@ def setup_common_training_handlers(
 ) -> None:
     """Helper method to setup trainer with common handlers (it also supports distributed configuration):
 
-        - :class:`~ignite.handlers.TerminateOnNan`
+        - :class:`~ignite.handlers.terminate_on_nan.TerminateOnNan`
         - handler to setup learning rate scheduling
-        - :class:`~ignite.handlers.ModelCheckpoint`
+        - :class:`~ignite.handlers.checkpoint.ModelCheckpoint`
         - :class:`~ignite.metrics.RunningAverage` on `update_function` output
         - Two progress bars on epochs and optionally on iterations
 
     Args:
-        trainer (Engine): trainer engine. Output of trainer's `update_function` should be a dictionary
+        trainer: trainer engine. Output of trainer's `update_function` should be a dictionary
             or sequence or a single tensor.
-        train_sampler (torch.utils.data.DistributedSampler, optional): Optional distributed sampler used to call
+        train_sampler: Optional distributed sampler used to call
             `set_epoch` method on epoch started event.
-        to_save (dict, optional): dictionary with objects to save in the checkpoint. This argument is passed to
-            :class:`~ignite.handlers.Checkpoint` instance.
-        save_every_iters (int, optional): saving interval. By default, `to_save` objects are stored
+        to_save: dictionary with objects to save in the checkpoint. This argument is passed to
+            :class:`~ignite.handlers.checkpoint.Checkpoint` instance.
+        save_every_iters: saving interval. By default, `to_save` objects are stored
             each 1000 iterations.
-        output_path (str, optional): output path to indicate where `to_save` objects are stored. Argument is mutually
+        output_path: output path to indicate where `to_save` objects are stored. Argument is mutually
             exclusive with ``save_handler``.
-        lr_scheduler (ParamScheduler or subclass of `torch.optim.lr_scheduler._LRScheduler`): learning rate scheduler
+        lr_scheduler: learning rate scheduler
             as native torch LRScheduler or ignite's parameter scheduler.
-        with_gpu_stats (bool, optional): if True, :class:`~ignite.contrib.metrics.handlers.GpuInfo` is attached to the
+        with_gpu_stats: if True, :class:`~ignite.contrib.metrics.GpuInfo` is attached to the
             trainer. This requires `pynvml` package to be installed.
-        output_names (list/tuple, optional): list of names associated with `update_function` output dictionary.
-        with_pbars (bool, optional): if True, two progress bars on epochs and optionally on iterations are attached.
+        output_names: list of names associated with `update_function` output dictionary.
+        with_pbars: if True, two progress bars on epochs and optionally on iterations are attached.
             Default, True.
-        with_pbar_on_iters (bool, optional): if True, a progress bar on iterations is attached to the trainer.
+        with_pbar_on_iters: if True, a progress bar on iterations is attached to the trainer.
             Default, True.
-        log_every_iters (int, optional): logging interval for :class:`~ignite.contrib.metrics.handlers.GpuInfo` and for
+        log_every_iters: logging interval for :class:`~ignite.contrib.metrics.GpuInfo` and for
             epoch-wise progress bar. Default, 100.
-        stop_on_nan (bool, optional): if True, :class:`~ignite.handlers.TerminateOnNan` handler is added to the trainer.
+        stop_on_nan: if True, :class:`~ignite.handlers.terminate_on_nan.TerminateOnNan` handler is added to the trainer.
             Default, True.
-        clear_cuda_cache (bool, optional): if True, `torch.cuda.empty_cache()` is called every end of epoch.
+        clear_cuda_cache: if True, `torch.cuda.empty_cache()` is called every end of epoch.
             Default, True.
-        save_handler (callable or :class:`~ignite.handlers.checkpoint.BaseSaveHandler`, optional): Method or callable
+        save_handler: Method or callable
             class to use to store ``to_save``. See :class:`~ignite.handlers.checkpoint.Checkpoint` for more details.
             Argument is mutually exclusive with ``output_path``.
-        **kwargs: optional keyword args to be passed to construct :class:`~ignite.handlers.checkpoint.Checkpoint`.
-        device (str of torch.device, optional): deprecated argument, it will be removed in v0.5.0.
+        kwargs: optional keyword args to be passed to construct :class:`~ignite.handlers.checkpoint.Checkpoint`.
+        device: deprecated argument, it will be removed in v0.5.0.
     """
     if device is not None:
         warnings.warn("Argument device is unused and deprecated. It will be removed in v0.5.0")
@@ -146,7 +149,7 @@ def _setup_common_training_handlers(
     to_save: Optional[Mapping] = None,
     save_every_iters: int = 1000,
     output_path: Optional[str] = None,
-    lr_scheduler: Optional[Union[ParamScheduler, _LRScheduler]] = None,
+    lr_scheduler: Optional[Union[ParamScheduler, PyTorchLRScheduler]] = None,
     with_gpu_stats: bool = False,
     output_names: Optional[Iterable[str]] = None,
     with_pbars: bool = True,
@@ -166,12 +169,10 @@ def _setup_common_training_handlers(
         trainer.add_event_handler(Events.ITERATION_COMPLETED, TerminateOnNan())
 
     if lr_scheduler is not None:
-        if isinstance(lr_scheduler, torch.optim.lr_scheduler._LRScheduler):
+        if isinstance(lr_scheduler, PyTorchLRScheduler):
             trainer.add_event_handler(
-                Events.ITERATION_COMPLETED, lambda engine: cast(_LRScheduler, lr_scheduler).step()
+                Events.ITERATION_COMPLETED, lambda engine: cast(PyTorchLRScheduler, lr_scheduler).step()
             )
-        elif isinstance(lr_scheduler, LRScheduler):
-            trainer.add_event_handler(Events.ITERATION_COMPLETED, lr_scheduler)
         else:
             trainer.add_event_handler(Events.ITERATION_STARTED, lr_scheduler)
 
@@ -234,7 +235,7 @@ def _setup_common_distrib_training_handlers(
     to_save: Optional[Mapping] = None,
     save_every_iters: int = 1000,
     output_path: Optional[str] = None,
-    lr_scheduler: Optional[Union[ParamScheduler, _LRScheduler]] = None,
+    lr_scheduler: Optional[Union[ParamScheduler, PyTorchLRScheduler]] = None,
     with_gpu_stats: bool = False,
     output_names: Optional[Iterable[str]] = None,
     with_pbars: bool = True,
@@ -356,15 +357,15 @@ def setup_tb_logging(
         - Evaluation metrics
 
     Args:
-        output_path (str): logging directory path
-        trainer (Engine): trainer engine
-        optimizers (torch.optim.Optimizer or dict of torch.optim.Optimizer, optional): single or dictionary of
+        output_path: logging directory path
+        trainer: trainer engine
+        optimizers: single or dictionary of
             torch optimizers. If a dictionary, keys are used as tags arguments for logging.
-        evaluators (Engine or dict of Engine, optional): single or dictionary of evaluators. If a dictionary,
+        evaluators: single or dictionary of evaluators. If a dictionary,
             keys are used as tags arguments for logging.
-        log_every_iters (int, optional): interval for loggers attached to iteration events. To log every iteration,
+        log_every_iters: interval for loggers attached to iteration events. To log every iteration,
             value can be set to 1 or None.
-        **kwargs: optional keyword args to be passed to construct the logger.
+        kwargs: optional keyword args to be passed to construct the logger.
 
     Returns:
         :class:`~ignite.contrib.handlers.tensorboard_logger.TensorboardLogger`
@@ -388,14 +389,14 @@ def setup_visdom_logging(
         - Evaluation metrics
 
     Args:
-        trainer (Engine): trainer engine
-        optimizers (torch.optim.Optimizer or dict of torch.optim.Optimizer, optional): single or dictionary of
+        trainer: trainer engine
+        optimizers: single or dictionary of
             torch optimizers. If a dictionary, keys are used as tags arguments for logging.
-        evaluators (Engine or dict of Engine, optional): single or dictionary of evaluators. If a dictionary,
+        evaluators: single or dictionary of evaluators. If a dictionary,
             keys are used as tags arguments for logging.
-        log_every_iters (int, optional): interval for loggers attached to iteration events. To log every iteration,
+        log_every_iters: interval for loggers attached to iteration events. To log every iteration,
             value can be set to 1 or None.
-        **kwargs: optional keyword args to be passed to construct the logger.
+        kwargs: optional keyword args to be passed to construct the logger.
 
     Returns:
         :class:`~ignite.contrib.handlers.visdom_logger.VisdomLogger`
@@ -419,14 +420,14 @@ def setup_mlflow_logging(
         - Evaluation metrics
 
     Args:
-        trainer (Engine): trainer engine
-        optimizers (torch.optim.Optimizer or dict of torch.optim.Optimizer, optional): single or dictionary of
+        trainer: trainer engine
+        optimizers: single or dictionary of
             torch optimizers. If a dictionary, keys are used as tags arguments for logging.
-        evaluators (Engine or dict of Engine, optional): single or dictionary of evaluators. If a dictionary,
+        evaluators: single or dictionary of evaluators. If a dictionary,
             keys are used as tags arguments for logging.
-        log_every_iters (int, optional): interval for loggers attached to iteration events. To log every iteration,
+        log_every_iters: interval for loggers attached to iteration events. To log every iteration,
             value can be set to 1 or None.
-        **kwargs: optional keyword args to be passed to construct the logger.
+        kwargs: optional keyword args to be passed to construct the logger.
 
     Returns:
         :class:`~ignite.contrib.handlers.mlflow_logger.MLflowLogger`
@@ -450,14 +451,14 @@ def setup_neptune_logging(
         - Evaluation metrics
 
     Args:
-        trainer (Engine): trainer engine
-        optimizers (torch.optim.Optimizer or dict of torch.optim.Optimizer, optional): single or dictionary of
+        trainer: trainer engine
+        optimizers: single or dictionary of
             torch optimizers. If a dictionary, keys are used as tags arguments for logging.
-        evaluators (Engine or dict of Engine, optional): single or dictionary of evaluators. If a dictionary,
+        evaluators: single or dictionary of evaluators. If a dictionary,
             keys are used as tags arguments for logging.
-        log_every_iters (int, optional): interval for loggers attached to iteration events. To log every iteration,
+        log_every_iters: interval for loggers attached to iteration events. To log every iteration,
             value can be set to 1 or None.
-        **kwargs: optional keyword args to be passed to construct the logger.
+        kwargs: optional keyword args to be passed to construct the logger.
 
     Returns:
         :class:`~ignite.contrib.handlers.neptune_logger.NeptuneLogger`
@@ -481,14 +482,14 @@ def setup_wandb_logging(
         - Evaluation metrics
 
     Args:
-        trainer (Engine): trainer engine
-        optimizers (torch.optim.Optimizer or dict of torch.optim.Optimizer, optional): single or dictionary of
+        trainer: trainer engine
+        optimizers: single or dictionary of
             torch optimizers. If a dictionary, keys are used as tags arguments for logging.
-        evaluators (Engine or dict of Engine, optional): single or dictionary of evaluators. If a dictionary,
+        evaluators: single or dictionary of evaluators. If a dictionary,
             keys are used as tags arguments for logging.
-        log_every_iters (int, optional): interval for loggers attached to iteration events. To log every iteration,
+        log_every_iters: interval for loggers attached to iteration events. To log every iteration,
             value can be set to 1 or None.
-        **kwargs: optional keyword args to be passed to construct the logger.
+        kwargs: optional keyword args to be passed to construct the logger.
 
     Returns:
         :class:`~ignite.contrib.handlers.wandb_logger.WandBLogger`
@@ -512,14 +513,14 @@ def setup_plx_logging(
         - Evaluation metrics
 
     Args:
-        trainer (Engine): trainer engine
-        optimizers (torch.optim.Optimizer or dict of torch.optim.Optimizer, optional): single or dictionary of
+        trainer: trainer engine
+        optimizers: single or dictionary of
             torch optimizers. If a dictionary, keys are used as tags arguments for logging.
-        evaluators (Engine or dict of Engine, optional): single or dictionary of evaluators. If a dictionary,
+        evaluators: single or dictionary of evaluators. If a dictionary,
             keys are used as tags arguments for logging.
-        log_every_iters (int, optional): interval for loggers attached to iteration events. To log every iteration,
+        log_every_iters: interval for loggers attached to iteration events. To log every iteration,
             value can be set to 1 or None.
-        **kwargs: optional keyword args to be passed to construct the logger.
+        kwargs: optional keyword args to be passed to construct the logger.
 
     Returns:
         :class:`~ignite.contrib.handlers.polyaxon_logger.PolyaxonLogger`
@@ -543,14 +544,14 @@ def setup_clearml_logging(
         - Evaluation metrics
 
     Args:
-        trainer (Engine): trainer engine
-        optimizers (torch.optim.Optimizer or dict of torch.optim.Optimizer, optional): single or dictionary of
+        trainer: trainer engine
+        optimizers: single or dictionary of
             torch optimizers. If a dictionary, keys are used as tags arguments for logging.
-        evaluators (Engine or dict of Engine, optional): single or dictionary of evaluators. If a dictionary,
+        evaluators: single or dictionary of evaluators. If a dictionary,
             keys are used as tags arguments for logging.
-        log_every_iters (int, optional): interval for loggers attached to iteration events. To log every iteration,
+        log_every_iters: interval for loggers attached to iteration events. To log every iteration,
             value can be set to 1 or None.
-        **kwargs: optional keyword args to be passed to construct the logger.
+        kwargs: optional keyword args to be passed to construct the logger.
 
     Returns:
         :class:`~ignite.contrib.handlers.clearml_logger.ClearMLLogger`
@@ -567,8 +568,7 @@ def setup_trains_logging(
     log_every_iters: int = 100,
     **kwargs: Any,
 ) -> ClearMLLogger:
-    """``setup_trains_logging`` was renamed to :func:`~ignite.contrib.engines.common.setup_clearml_logging`.
-    """
+    """``setup_trains_logging`` was renamed to :func:`~ignite.contrib.engines.common.setup_clearml_logging`."""
     warnings.warn("setup_trains_logging was renamed to setup_clearml_logging.")
     return setup_clearml_logging(trainer, optimizers, evaluators, log_every_iters, **kwargs)
 
@@ -584,6 +584,7 @@ def gen_save_best_models_by_val_score(
     n_saved: int = 3,
     trainer: Optional[Engine] = None,
     tag: str = "val",
+    score_sign: float = 1.0,
     **kwargs: Any,
 ) -> Checkpoint:
     """Method adds a handler to ``evaluator`` to save ``n_saved`` of best models based on the metric
@@ -592,31 +593,33 @@ def gen_save_best_models_by_val_score(
     ``save_handler``.
 
     Args:
-        save_handler (callable or :class:`~ignite.handlers.checkpoint.BaseSaveHandler`): Method or callable class to
+        save_handler: Method or callable class to
             use to save engine and other provided objects. Function receives two objects: checkpoint as a dictionary
             and filename. If ``save_handler`` is callable class, it can
             inherit of :class:`~ignite.handlers.checkpoint.BaseSaveHandler` and optionally implement ``remove`` method
             to keep a fixed number of saved checkpoints. In case if user needs to save engine's checkpoint on a disk,
             ``save_handler`` can be defined with :class:`~ignite.handlers.DiskSaver`.
-        evaluator (Engine): evaluation engine used to provide the score
-        models (nn.Module or Mapping): model or dictionary with the object to save. Objects should have
+        evaluator: evaluation engine used to provide the score
+        models: model or dictionary with the object to save. Objects should have
             implemented ``state_dict`` and ``load_state_dict`` methods.
-        metric_name (str): metric name to use for score evaluation. This metric should be present in
+        metric_name: metric name to use for score evaluation. This metric should be present in
             `evaluator.state.metrics`.
-        n_saved (int, optional): number of best models to store
-        trainer (Engine, optional): trainer engine to fetch the epoch when saving the best model.
-        tag (str, optional): score name prefix: `{tag}_{metric_name}`. By default, tag is "val".
-        **kwargs: optional keyword args to be passed to construct :class:`~ignite.handlers.checkpoint.Checkpoint`.
+        n_saved: number of best models to store
+        trainer: trainer engine to fetch the epoch when saving the best model.
+        tag: score name prefix: `{tag}_{metric_name}`. By default, tag is "val".
+        score_sign: sign of the score: 1.0 or -1.0. For error-like metrics, e.g. smaller is better,
+            a negative score sign should be used (objects with larger score are retained). Default, 1.0.
+        kwargs: optional keyword args to be passed to construct :class:`~ignite.handlers.checkpoint.Checkpoint`.
 
     Returns:
-        A :class:`~ignite.handlers.Checkpoint` handler.
+        A :class:`~ignite.handlers.checkpoint.Checkpoint` handler.
     """
     global_step_transform = None
     if trainer is not None:
         global_step_transform = global_step_from_engine(trainer)
 
     if isinstance(models, nn.Module):
-        to_save = {"model": models}  # type: Dict[str, nn.Module]
+        to_save: Dict[str, nn.Module] = {"model": models}
     else:
         to_save = models
 
@@ -627,7 +630,7 @@ def gen_save_best_models_by_val_score(
         n_saved=n_saved,
         global_step_transform=global_step_transform,
         score_name=f"{tag}_{metric_name.lower()}",
-        score_function=Checkpoint.get_default_score_fn(metric_name),
+        score_function=get_default_score_fn(metric_name, score_sign=score_sign),
         **kwargs,
     )
     evaluator.add_event_handler(Events.COMPLETED, best_model_handler)
@@ -643,6 +646,7 @@ def save_best_model_by_val_score(
     n_saved: int = 3,
     trainer: Optional[Engine] = None,
     tag: str = "val",
+    score_sign: float = 1.0,
     **kwargs: Any,
 ) -> Checkpoint:
     """Method adds a handler to ``evaluator`` to save on a disk ``n_saved`` of best models based on the metric
@@ -650,18 +654,21 @@ def save_best_model_by_val_score(
     Models with highest metric value will be retained.
 
     Args:
-        output_path (str): output path to indicate where to save best models
-        evaluator (Engine): evaluation engine used to provide the score
-        model (nn.Module): model to store
-        metric_name (str): metric name to use for score evaluation. This metric should be present in
+        output_path: output path to indicate where to save best models
+        evaluator: evaluation engine used to provide the score
+        model: model to store
+        metric_name: metric name to use for score evaluation. This metric should be present in
             `evaluator.state.metrics`.
-        n_saved (int, optional): number of best models to store
-        trainer (Engine, optional): trainer engine to fetch the epoch when saving the best model.
-        tag (str, optional): score name prefix: `{tag}_{metric_name}`. By default, tag is "val".
-        **kwargs: optional keyword args to be passed to construct :class:`~ignite.handlers.checkpoint.Checkpoint`.
+        n_saved: number of best models to store
+        trainer: trainer engine to fetch the epoch when saving the best model.
+        tag: score name prefix: `{tag}_{metric_name}`. By default, tag is "val".
+        score_sign: sign of the score: 1.0 or -1.0. For error-like metrics, e.g. smaller is better,
+            a negative score sign should be used (objects with larger score are retained). Default, 1.0.
+
+        kwargs: optional keyword args to be passed to construct :class:`~ignite.handlers.checkpoint.Checkpoint`.
 
     Returns:
-        A :class:`~ignite.handlers.Checkpoint` handler.
+        A :class:`~ignite.handlers.checkpoint.Checkpoint` handler.
     """
     return gen_save_best_models_by_val_score(
         save_handler=DiskSaver(dirname=output_path, require_empty=False),
@@ -671,27 +678,36 @@ def save_best_model_by_val_score(
         n_saved=n_saved,
         trainer=trainer,
         tag=tag,
+        score_sign=score_sign,
         **kwargs,
     )
 
 
 def add_early_stopping_by_val_score(
-    patience: int, evaluator: Engine, trainer: Engine, metric_name: str
+    patience: int,
+    evaluator: Engine,
+    trainer: Engine,
+    metric_name: str,
+    score_sign: float = 1.0,
 ) -> EarlyStopping:
     """Method setups early stopping handler based on the score (named by `metric_name`) provided by `evaluator`.
     Metric value should increase in order to keep training and not early stop.
 
     Args:
-        patience (int): number of events to wait if no improvement and then stop the training.
-        evaluator (Engine): evaluation engine used to provide the score
-        trainer (Engine): trainer engine to stop the run if no improvement.
-        metric_name (str): metric name to use for score evaluation. This metric should be present in
+        patience: number of events to wait if no improvement and then stop the training.
+        evaluator: evaluation engine used to provide the score
+        trainer: trainer engine to stop the run if no improvement.
+        metric_name: metric name to use for score evaluation. This metric should be present in
             `evaluator.state.metrics`.
+        score_sign: sign of the score: 1.0 or -1.0. For error-like metrics, e.g. smaller is better,
+            a negative score sign should be used (objects with larger score are retained). Default, 1.0.
 
     Returns:
-        A :class:`~ignite.handlers.EarlyStopping` handler.
+        A :class:`~ignite.handlers.early_stopping.EarlyStopping` handler.
     """
-    es_handler = EarlyStopping(patience=patience, score_function=get_default_score_fn(metric_name), trainer=trainer)
+    es_handler = EarlyStopping(
+        patience=patience, score_function=get_default_score_fn(metric_name, score_sign=score_sign), trainer=trainer
+    )
     evaluator.add_event_handler(Events.COMPLETED, es_handler)
 
     return es_handler
