@@ -1,23 +1,22 @@
 from ignite.contrib.handlers.clearml_logger import ClearMLLogger
 from ignite.contrib.handlers.custom_events import CustomPeriodicEvent
-from ignite.contrib.handlers.lr_finder import FastaiLRFinder
 from ignite.contrib.handlers.mlflow_logger import MLflowLogger
 from ignite.contrib.handlers.neptune_logger import NeptuneLogger
-from ignite.contrib.handlers.param_scheduler import (
+from ignite.contrib.handlers.polyaxon_logger import PolyaxonLogger
+from ignite.contrib.handlers.tensorboard_logger import TensorboardLogger
+from ignite.contrib.handlers.tqdm_logger import ProgressBar
+
+from ignite.contrib.handlers.visdom_logger import VisdomLogger
+from ignite.contrib.handlers.wandb_logger import WandBLogger
+from ignite.handlers import EpochOutputStore, global_step_from_engine  # ref  # ref
+from ignite.handlers.lr_finder import FastaiLRFinder
+from ignite.handlers.param_scheduler import (
     ConcatScheduler,
     CosineAnnealingScheduler,
+    create_lr_scheduler_with_warmup,
     LinearCyclicalScheduler,
     LRScheduler,
     ParamGroupScheduler,
     PiecewiseLinear,
-    create_lr_scheduler_with_warmup,
 )
-from ignite.contrib.handlers.polyaxon_logger import PolyaxonLogger
-from ignite.contrib.handlers.stores import EpochOutputStore
-from ignite.contrib.handlers.tensorboard_logger import TensorboardLogger
-from ignite.contrib.handlers.time_profilers import BasicTimeProfiler, HandlersTimeProfiler
-from ignite.contrib.handlers.tqdm_logger import ProgressBar
-from ignite.contrib.handlers.trains_logger import TrainsLogger
-from ignite.contrib.handlers.visdom_logger import VisdomLogger
-from ignite.contrib.handlers.wandb_logger import WandBLogger
-from ignite.handlers import global_step_from_engine  # ref
+from ignite.handlers.time_profilers import BasicTimeProfiler, HandlersTimeProfiler
