@@ -1,4 +1,4 @@
-import numbers
+"""ClearML logger and its helper handlers."""
 import os
 import tempfile
 import warnings
@@ -7,8 +7,6 @@ from datetime import datetime
 from enum import Enum
 from typing import Any, Callable, DefaultDict, List, Mapping, Optional, Tuple, Type, Union
 
-import torch
-from torch.nn import Module
 from torch.optim import Optimizer
 
 import ignite.distributed as idist
@@ -16,7 +14,7 @@ from ignite.contrib.handlers.base_logger import (
     BaseLogger,
     BaseOptimizerParamsHandler,
     BaseOutputHandler,
-    BaseWeightsHistHandler,
+    BaseWeightsHandler,
     BaseWeightsScalarHandler,
 )
 from ignite.engine import Engine, Events
@@ -49,18 +47,11 @@ class ClearMLLogger(BaseLogger):
         clearml-init
 
     Args:
-        project_name (str): The name of the project in which the experiment will be created. If the project
-            does not exist, it is created. If ``project_name`` is ``None``, the repository name is used. (Optional)
-        task_name (str): The name of Task (experiment). If ``task_name`` is ``None``, the Python experiment
-            script's file name is used. (Optional)
-        task_type (str): Optional. The task type. Valid values are:
-            - ``TaskTypes.training`` (Default)
-            - ``TaskTypes.train``
-            - ``TaskTypes.testing``
-            - ``TaskTypes.inference``
+        kwargs: Keyword arguments accepted from ``Task.init`` method.
+            All arguments are optional. If a ClearML Task has already been created,
+            kwargs will be ignored and the current ClearML Task will be used.
 
     Examples:
-
         .. code-block:: python
 
             from ignite.contrib.handlers.clearml_logger import *
@@ -119,40 +110,25 @@ class ClearMLLogger(BaseLogger):
 
     """
 
-    def __init__(self, *_: Any, **kwargs: Any) -> None:
+    def __init__(self, **kwargs: Any):
         try:
             from clearml import Task
             from clearml.binding.frameworks.tensorflow_bind import WeightsGradientHistHelper
         except ImportError:
-            try:
-                # Backwards-compatibility for legacy Trains SDK
-                from trains import Task
-                from trains.binding.frameworks.tensorflow_bind import WeightsGradientHistHelper
-            except ImportError:
-                raise RuntimeError(
-                    "This contrib module requires clearml to be installed. "
-                    "You may install clearml using: \n pip install clearml \n"
-                )
+            raise ModuleNotFoundError(
+                "This contrib module requires clearml to be installed. "
+                "You may install clearml using: \n pip install clearml \n"
+            )
 
         experiment_kwargs = {k: v for k, v in kwargs.items() if k not in ("project_name", "task_name", "task_type")}
 
         if self.bypass_mode():
             warnings.warn("ClearMLSaver: running in bypass mode")
 
-            class _Stub(object):
-                def __call__(self, *_: Any, **__: Any) -> "_Stub":
-                    return self
+        # Try to retrieve current the ClearML Task before trying to create a new one
+        self._task = Task.current_task()
 
-                def __getattr__(self, attr: str) -> "_Stub":
-                    if attr in ("name", "id"):
-                        return ""  # type: ignore[return-value]
-                    return self
-
-                def __setattr__(self, attr: str, val: Any) -> None:
-                    pass
-
-            self._task = _Stub()
-        else:
+        if self._task is None:
             self._task = Task.init(
                 project_name=kwargs.get("project_name"),
                 task_name=kwargs.get("task_name"),
@@ -162,18 +138,25 @@ class ClearMLLogger(BaseLogger):
 
         self.clearml_logger = self._task.get_logger()
 
-        self.grad_helper = WeightsGradientHistHelper(logger=self.clearml_logger)
+        self.grad_helper = WeightsGradientHistHelper(logger=self.clearml_logger, report_freq=1)
 
     @classmethod
     def set_bypass_mode(cls, bypass: bool) -> None:
         """
-        Will bypass all outside communication, and will drop all logs.
+        Set ``clearml.Task`` to offline mode.
+        Will bypass all outside communication, and will save all data and logs to a local session folder.
         Should only be used in "standalone mode", when there is no access to the *clearml-server*.
 
         Args:
             bypass: If ``True``, all outside communication is skipped.
+                Data and logs will be stored in a local session folder.
+                For more information, please refer to `ClearML docs
+                <https://clear.ml/docs/latest/docs/clearml_sdk/task_sdk/#offline-mode>`_.
         """
+        from clearml import Task
+
         setattr(cls, "_bypass", bypass)
+        Task.set_offline(offline_mode=bypass)
 
     @classmethod
     def bypass_mode(cls) -> bool:
@@ -183,12 +166,32 @@ class ClearMLLogger(BaseLogger):
         Note:
             `GITHUB_ACTIONS` env will automatically set bypass_mode to ``True``
             unless overridden specifically with ``ClearMLLogger.set_bypass_mode(False)``.
+            For more information, please refer to `ClearML docs
+            <https://clear.ml/docs/latest/docs/clearml_sdk/task_sdk/#offline-mode>`_.
 
         Return:
-            If True, all outside communication is skipped.
+            If True, ``clearml.Task`` is on offline mode, and all outside communication is skipped.
         """
         return getattr(cls, "_bypass", bool(os.environ.get("CI")))
 
+    def __getattr__(self, attr: Any) -> Any:
+        """
+        Calls the corresponding method of ``clearml.Logger``.
+
+        Args:
+            attr: methods of the ``clearml.Logger`` class.
+        """
+        return getattr(self.clearml_logger, attr)
+
+    def get_task(self) -> Any:
+        """
+        Returns the task context that the logger is reporting.
+
+        Return:
+            Returns the current task, equivalent to ``clearml.Task.current_task()``.
+        """
+        return self._task
+
     def close(self) -> None:
         self.clearml_logger.flush()
 
@@ -202,8 +205,22 @@ class ClearMLLogger(BaseLogger):
 class OutputHandler(BaseOutputHandler):
     """Helper handler to log engine's output and/or metrics
 
-    Examples:
+    Args:
+        tag: common title for all produced plots. For example, "training"
+        metric_names: list of metric names to plot or a string "all" to plot all available
+            metrics.
+        output_transform: output transform function to prepare `engine.state.output` as a number.
+            For example, `output_transform = lambda output: output`
+            This function can also return a dictionary, e.g `{"loss": loss1, "another_loss": loss2}` to label the plot
+            with corresponding keys.
+        global_step_transform: global step transform function to output a desired global step.
+            Input of the function is `(engine, event_name)`. Output of function should be an integer.
+            Default is None, global_step based on attached engine. If provided,
+            uses function output as global_step. To setup global step from another engine, please use
+            :meth:`~ignite.contrib.handlers.clearml_logger.global_step_from_engine`.
+        state_attributes: list of attributes of the ``trainer.state`` to plot.
 
+    Examples:
         .. code-block:: python
 
             from ignite.contrib.handlers.clearml_logger import *
@@ -269,28 +286,30 @@ class OutputHandler(BaseOutputHandler):
                 global_step_transform=global_step_transform
             )
 
-    Args:
-        tag (str): common title for all produced plots. For example, "training"
-        metric_names (list of str, optional): list of metric names to plot or a string "all" to plot all available
-            metrics.
-        output_transform (callable, optional): output transform function to prepare `engine.state.output` as a number.
-            For example, `output_transform = lambda output: output`
-            This function can also return a dictionary, e.g `{"loss": loss1, "another_loss": loss2}` to label the plot
-            with corresponding keys.
-        global_step_transform (callable, optional): global step transform function to output a desired global step.
-            Input of the function is `(engine, event_name)`. Output of function should be an integer.
-            Default is None, global_step based on attached engine. If provided,
-            uses function output as global_step. To setup global step from another engine, please use
-            :meth:`~ignite.contrib.handlers.clearml_logger.global_step_from_engine`.
+        Another example where the State Attributes ``trainer.state.alpha`` and ``trainer.state.beta``
+        are also logged along with the NLL and Accuracy after each iteration:
 
-    Note:
-        Example of `global_step_transform`:
+        .. code-block:: python
+
+            clearml_logger.attach(
+                trainer,
+                log_handler=OutputHandler(
+                    tag="training",
+                    metric_names=["nll", "accuracy"],
+                    state_attributes=["alpha", "beta"],
+                ),
+                event_name=Events.ITERATION_COMPLETED
+            )
+
+        Example of `global_step_transform`
 
         .. code-block:: python
 
             def global_step_transform(engine, event_name):
                 return engine.state.get_event_attrib_value(event_name)
 
+    .. versionchanged:: 0.4.7
+        accepts an optional list of `state_attributes`
     """
 
     def __init__(
@@ -298,18 +317,21 @@ class OutputHandler(BaseOutputHandler):
         tag: str,
         metric_names: Optional[List[str]] = None,
         output_transform: Optional[Callable] = None,
-        global_step_transform: Optional[Callable] = None,
-    ) -> None:
-        super(OutputHandler, self).__init__(tag, metric_names, output_transform, global_step_transform)
+        global_step_transform: Optional[Callable[[Engine, Union[str, Events]], int]] = None,
+        state_attributes: Optional[List[str]] = None,
+    ):
+        super(OutputHandler, self).__init__(
+            tag, metric_names, output_transform, global_step_transform, state_attributes
+        )
 
     def __call__(self, engine: Engine, logger: ClearMLLogger, event_name: Union[str, Events]) -> None:
 
         if not isinstance(logger, ClearMLLogger):
             raise RuntimeError("Handler OutputHandler works only with ClearMLLogger")
 
-        metrics = self._setup_output_metrics(engine)
+        metrics = self._setup_output_metrics_state_attrs(engine)
 
-        global_step = self.global_step_transform(engine, event_name)  # type: ignore[misc]
+        global_step = self.global_step_transform(engine, event_name)
 
         if not isinstance(global_step, int):
             raise TypeError(
@@ -318,22 +340,24 @@ class OutputHandler(BaseOutputHandler):
             )
 
         for key, value in metrics.items():
-            if isinstance(value, numbers.Number) or isinstance(value, torch.Tensor) and value.ndimension() == 0:
-                logger.clearml_logger.report_scalar(title=self.tag, series=key, iteration=global_step, value=value)
-            elif isinstance(value, torch.Tensor) and value.ndimension() == 1:
-                for i, v in enumerate(value):
-                    logger.clearml_logger.report_scalar(
-                        title=f"{self.tag}/{key}", series=str(i), iteration=global_step, value=v.item()
-                    )
-            else:
-                warnings.warn(f"ClearMLLogger output_handler can not log metrics value type {type(value)}")
+            if len(key) == 2:
+                logger.clearml_logger.report_scalar(title=key[0], series=key[1], iteration=global_step, value=value)
+            elif len(key) == 3:
+                logger.clearml_logger.report_scalar(
+                    title=f"{key[0]}/{key[1]}", series=key[2], iteration=global_step, value=value
+                )
 
 
 class OptimizerParamsHandler(BaseOptimizerParamsHandler):
     """Helper handler to log optimizer parameters
 
-    Examples:
+    Args:
+        optimizer: torch optimizer or any object with attribute ``param_groups``
+            as a sequence.
+        param_name: parameter name
+        tag: common title for all produced plots. For example, "generator"
 
+    Examples:
         .. code-block:: python
 
             from ignite.contrib.handlers.clearml_logger import *
@@ -357,15 +381,9 @@ class OptimizerParamsHandler(BaseOptimizerParamsHandler):
                 event_name=Events.ITERATION_STARTED,
                 optimizer=optimizer
             )
-
-    Args:
-        optimizer (torch.optim.Optimizer or object): torch optimizer or any object with attribute ``param_groups``
-            as a sequence.
-        param_name (str): parameter name
-        tag (str, optional): common title for all produced plots. For example, "generator"
     """
 
-    def __init__(self, optimizer: Optimizer, param_name: str = "lr", tag: Optional[str] = None) -> None:
+    def __init__(self, optimizer: Optimizer, param_name: str = "lr", tag: Optional[str] = None):
         super(OptimizerParamsHandler, self).__init__(optimizer, param_name, tag)
 
     def __call__(self, engine: Engine, logger: ClearMLLogger, event_name: Union[str, Events]) -> None:
@@ -386,11 +404,22 @@ class OptimizerParamsHandler(BaseOptimizerParamsHandler):
 
 class WeightsScalarHandler(BaseWeightsScalarHandler):
     """Helper handler to log model's weights as scalars.
-    Handler iterates over named parameters of the model, applies reduction function to each parameter
-    produce a scalar and then logs the scalar.
+    Handler, upon construction, iterates over named parameters of the model and keep
+    reference to ones permitted by `whitelist`. Then at every call, applies
+    reduction function to each parameter, produces a scalar and logs it.
+
+    Args:
+        model: model to log weights
+        reduction: function to reduce parameters into scalar
+        tag: common title for all produced plots. For example, "generator"
+        whitelist: specific weights to log. Should be list of model's submodules
+            or parameters names, or a callable which gets weight along with its name
+            and determines if it should be logged. Names should be fully-qualified.
+            For more information please refer to `PyTorch docs
+            <https://pytorch.org/docs/stable/generated/torch.nn.Module.html#torch.nn.Module.get_submodule>`_.
+            If not given, all of model's weights are logged.
 
     Examples:
-
         .. code-block:: python
 
             from ignite.contrib.handlers.clearml_logger import *
@@ -409,16 +438,48 @@ class WeightsScalarHandler(BaseWeightsScalarHandler):
                 log_handler=WeightsScalarHandler(model, reduction=torch.norm)
             )
 
-    Args:
-        model (torch.nn.Module): model to log weights
-        reduction (callable): function to reduce parameters into scalar
-        tag (str, optional): common title for all produced plots. For example, "generator"
+        .. code-block:: python
 
+            from ignite.contrib.handlers.clearml_logger import *
+
+            clearml_logger = ClearMLLogger(
+                project_name="pytorch-ignite-integration",
+                task_name="cnn-mnist"
+            )
+
+            # Log only `fc` weights
+            clearml_logger.attach(
+                trainer,
+                event_name=Events.ITERATION_COMPLETED,
+                log_handler=WeightsScalarHandler(
+                    model,
+                    whitelist=['fc']
+                )
+            )
+
+        .. code-block:: python
+
+            from ignite.contrib.handlers.clearml_logger import *
+
+            clearml_logger = ClearMLLogger(
+                project_name="pytorch-ignite-integration",
+                task_name="cnn-mnist"
+            )
+
+            # Log weights which have `bias` in their names
+            def has_bias_in_name(n, p):
+                return 'bias' in n
+
+            clearml_logger.attach(
+                trainer,
+                event_name=Events.ITERATION_COMPLETED,
+                log_handler=WeightsScalarHandler(model, whitelist=has_bias_in_name)
+            )
+
+    ..  versionchanged:: 0.4.9
+        optional argument `whitelist` added.
     """
 
-    def __init__(self, model: Module, reduction: Callable = torch.norm, tag: Optional[str] = None) -> None:
-        super(WeightsScalarHandler, self).__init__(model, reduction, tag=tag)
-
     def __call__(self, engine: Engine, logger: ClearMLLogger, event_name: Union[str, Events]) -> None:
 
         if not isinstance(logger, ClearMLLogger):
@@ -426,9 +487,7 @@ class WeightsScalarHandler(BaseWeightsScalarHandler):
 
         global_step = engine.state.get_event_attrib_value(event_name)
         tag_prefix = f"{self.tag}/" if self.tag else ""
-        for name, p in self.model.named_parameters():
-            if p.grad is None:
-                continue
+        for name, p in self.weights:
 
             title_name, _, series_name = name.partition(".")
             logger.clearml_logger.report_scalar(
@@ -439,11 +498,20 @@ class WeightsScalarHandler(BaseWeightsScalarHandler):
             )
 
 
-class WeightsHistHandler(BaseWeightsHistHandler):
+class WeightsHistHandler(BaseWeightsHandler):
     """Helper handler to log model's weights as histograms.
 
-    Examples:
+    Args:
+        model: model to log weights
+        tag: common title for all produced plots. For example, 'generator'
+        whitelist: specific weights to log. Should be list of model's submodules
+            or parameters names, or a callable which gets weight along with its name
+            and determines if it should be logged. Names should be fully-qualified.
+            For more information please refer to `PyTorch docs
+            <https://pytorch.org/docs/stable/generated/torch.nn.Module.html#torch.nn.Module.get_submodule>`_.
+            If not given, all of model's weights are logged.
 
+    Examples:
         .. code-block:: python
 
             from ignite.contrib.handlers.clearml_logger import *
@@ -462,24 +530,55 @@ class WeightsHistHandler(BaseWeightsHistHandler):
                 log_handler=WeightsHistHandler(model)
             )
 
-    Args:
-        model (torch.nn.Module): model to log weights
-        tag (str, optional): common title for all produced plots. For example, 'generator'
+        .. code-block:: python
 
+            from ignite.contrib.handlers.clearml_logger import *
+
+            clearml_logger = ClearMLLogger(
+                project_name="pytorch-ignite-integration",
+                task_name="cnn-mnist"
+            )
+
+            # Log weights of `fc` layer
+            weights = ['fc']
+
+            # Attach the logger to the trainer to log weights norm after each iteration
+            clearml_logger.attach(
+                trainer,
+                event_name=Events.ITERATION_COMPLETED,
+                log_handler=WeightsHistHandler(model, whitelist=weights)
+            )
+
+        .. code-block:: python
+
+            from ignite.contrib.handlers.clearml_logger import *
+
+            clearml_logger = ClearMLLogger(
+                project_name="pytorch-ignite-integration",
+                task_name="cnn-mnist"
+            )
+
+            # Log weights which name include 'conv'.
+            weight_selector = lambda name, p: 'conv' in name
+
+            # Attach the logger to the trainer to log weights norm after each iteration
+            clearml_logger.attach(
+                trainer,
+                event_name=Events.ITERATION_COMPLETED,
+                log_handler=WeightsHistHandler(model, whitelist=weight_selector)
+            )
+
+    ..  versionchanged:: 0.4.9
+        optional argument `whitelist` added.
     """
 
-    def __init__(self, model: Module, tag: Optional[str] = None) -> None:
-        super(WeightsHistHandler, self).__init__(model, tag=tag)
-
     def __call__(self, engine: Engine, logger: ClearMLLogger, event_name: Union[str, Events]) -> None:
         if not isinstance(logger, ClearMLLogger):
             raise RuntimeError("Handler 'WeightsHistHandler' works only with ClearMLLogger")
 
         global_step = engine.state.get_event_attrib_value(event_name)
         tag_prefix = f"{self.tag}/" if self.tag else ""
-        for name, p in self.model.named_parameters():
-            if p.grad is None:
-                continue
+        for name, p in self.weights:
 
             title_name, _, series_name = name.partition(".")
 
@@ -487,17 +586,28 @@ class WeightsHistHandler(BaseWeightsHistHandler):
                 title=f"{tag_prefix}weights_{title_name}",
                 series=series_name,
                 step=global_step,
-                hist_data=p.grad.detach().cpu().numpy(),
+                hist_data=p.data.cpu().numpy(),
             )
 
 
 class GradsScalarHandler(BaseWeightsScalarHandler):
     """Helper handler to log model's gradients as scalars.
-    Handler iterates over the gradients of named parameters of the model, applies reduction function to each parameter
-    produce a scalar and then logs the scalar.
+    Handler, upon construction, iterates over named parameters of the model and keep
+    reference to ones permitted by the `whitelist`. Then at every call, applies
+    reduction function to each parameter's gradient, produces a scalar and logs it.
+
+    Args:
+        model: model to log weights
+        reduction: function to reduce parameters into scalar
+        tag: common title for all produced plots. For example, "generator"
+        whitelist: specific gradients to log. Should be list of model's submodules
+            or parameters names, or a callable which gets weight along with its name
+            and determines if its gradient should be logged. Names should be
+            fully-qualified. For more information please refer to `PyTorch docs
+            <https://pytorch.org/docs/stable/generated/torch.nn.Module.html#torch.nn.Module.get_submodule>`_.
+            If not given, all of model's gradients are logged.
 
     Examples:
-
         .. code-block:: python
 
             from ignite.contrib.handlers.clearml_logger import *
@@ -516,23 +626,56 @@ class GradsScalarHandler(BaseWeightsScalarHandler):
                 log_handler=GradsScalarHandler(model, reduction=torch.norm)
             )
 
-    Args:
-        model (torch.nn.Module): model to log weights
-        reduction (callable): function to reduce parameters into scalar
-        tag (str, optional): common title for all produced plots. For example, "generator"
+        .. code-block:: python
 
+            from ignite.contrib.handlers.clearml_logger import *
+
+            clearml_logger = ClearMLLogger(
+                project_name="pytorch-ignite-integration",
+                task_name="cnn-mnist"
+            )
+
+            # Log gradient of `base`
+            clearml_logger.attach(
+                trainer,
+                event_name=Events.ITERATION_COMPLETED,
+                log_handler=GradsScalarHandler(
+                    model,
+                    reduction=torch.norm,
+                    whitelist=['base']
+                )
+            )
+
+        .. code-block:: python
+
+            from ignite.contrib.handlers.clearml_logger import *
+
+            clearml_logger = ClearMLLogger(
+                project_name="pytorch-ignite-integration",
+                task_name="cnn-mnist"
+            )
+
+            # Log gradient of weights which belong to a `fc` layer
+            def is_in_fc_layer(n, p):
+                return 'fc' in n
+
+            clearml_logger.attach(
+                trainer,
+                event_name=Events.ITERATION_COMPLETED,
+                log_handler=GradsScalarHandler(model, whitelist=is_in_fc_layer)
+            )
+
+    ..  versionchanged:: 0.4.9
+        optional argument `whitelist` added.
     """
 
-    def __init__(self, model: Module, reduction: Callable = torch.norm, tag: Optional[str] = None) -> None:
-        super(GradsScalarHandler, self).__init__(model, reduction, tag=tag)
-
     def __call__(self, engine: Engine, logger: ClearMLLogger, event_name: Union[str, Events]) -> None:
         if not isinstance(logger, ClearMLLogger):
             raise RuntimeError("Handler GradsScalarHandler works only with ClearMLLogger")
 
         global_step = engine.state.get_event_attrib_value(event_name)
         tag_prefix = f"{self.tag}/" if self.tag else ""
-        for name, p in self.model.named_parameters():
+        for name, p in self.weights:
             if p.grad is None:
                 continue
 
@@ -540,16 +683,25 @@ class GradsScalarHandler(BaseWeightsScalarHandler):
             logger.clearml_logger.report_scalar(
                 title=f"{tag_prefix}grads_{self.reduction.__name__}/{title_name}",
                 series=series_name,
-                value=self.reduction(p.data),
+                value=self.reduction(p.grad),
                 iteration=global_step,
             )
 
 
-class GradsHistHandler(BaseWeightsHistHandler):
+class GradsHistHandler(BaseWeightsHandler):
     """Helper handler to log model's gradients as histograms.
 
-    Examples:
+    Args:
+        model: model to log weights
+        tag: common title for all produced plots. For example, 'generator'
+        whitelist: specific gradients to log. Should be list of model's submodules
+            or parameters names, or a callable which gets weight along with its name
+            and determines if its gradient should be logged. Names should be
+            fully-qualified. For more information please refer to `PyTorch docs
+            <https://pytorch.org/docs/stable/generated/torch.nn.Module.html#torch.nn.Module.get_submodule>`_.
+            If not given, all of model's gradients are logged.
 
+    Examples:
         .. code-block:: python
 
             from ignite.contrib.handlers.clearml_logger import *
@@ -568,32 +720,61 @@ class GradsHistHandler(BaseWeightsHistHandler):
                 log_handler=GradsHistHandler(model)
             )
 
-    Args:
-        model (torch.nn.Module): model to log weights
-        tag (str, optional): common title for all produced plots. For example, 'generator'
+        .. code-block:: python
 
+            from ignite.contrib.handlers.clearml_logger import *
+
+            clearml_logger = ClearMLLogger(
+                project_name="pytorch-ignite-integration",
+                task_name="cnn-mnist"
+            )
+
+            # Log gradient of `fc.bias`
+            clearml_logger.attach(
+                trainer,
+                event_name=Events.ITERATION_COMPLETED,
+                log_handler=GradsHistHandler(model, whitelist=['fc.bias'])
+            )
+
+        .. code-block:: python
+
+            from ignite.contrib.handlers.clearml_logger import *
+
+            clearml_logger = ClearMLLogger(
+                project_name="pytorch-ignite-integration",
+                task_name="cnn-mnist"
+            )
+
+            # Log gradient of weights which have shape (2, 1)
+            def has_shape_2_1(n, p):
+                return p.shape == (2,1)
+
+            clearml_logger.attach(
+                trainer,
+                event_name=Events.ITERATION_COMPLETED,
+                log_handler=GradsHistHandler(model, whitelist=has_shape_2_1)
+            )
+
+    ..  versionchanged:: 0.4.9
+            optional argument `whitelist` added.
     """
 
-    def __init__(self, model: Module, tag: Optional[str] = None) -> None:
-        super(GradsHistHandler, self).__init__(model, tag=tag)
-
     def __call__(self, engine: Engine, logger: ClearMLLogger, event_name: Union[str, Events]) -> None:
         if not isinstance(logger, ClearMLLogger):
             raise RuntimeError("Handler 'GradsHistHandler' works only with ClearMLLogger")
 
         global_step = engine.state.get_event_attrib_value(event_name)
         tag_prefix = f"{self.tag}/" if self.tag else ""
-        for name, p in self.model.named_parameters():
+        for name, p in self.weights:
             if p.grad is None:
                 continue
 
             title_name, _, series_name = name.partition(".")
-
             logger.grad_helper.add_histogram(
                 title=f"{tag_prefix}grads_{title_name}",
                 series=series_name,
                 step=global_step,
-                hist_data=p.grad.detach().cpu().numpy(),
+                hist_data=p.grad.cpu().numpy(),
             )
 
 
@@ -602,16 +783,15 @@ class ClearMLSaver(DiskSaver):
     Handler that saves input checkpoint as ClearML artifacts
 
     Args:
-        logger (ClearMLLogger, optional): An instance of :class:`~ignite.contrib.handlers.clearml_logger.ClearMLLogger`,
+        logger: An instance of :class:`~ignite.contrib.handlers.clearml_logger.ClearMLLogger`,
             ensuring a valid ClearML ``Task`` has been initialized. If not provided, and a ClearML Task
             has not been manually initialized, a runtime error will be raised.
-        output_uri (str, optional): The default location for output models and other artifacts uploaded by ClearML. For
+        output_uri: The default location for output models and other artifacts uploaded by ClearML. For
             more information, see ``clearml.Task.init``.
-        dirname (str, optional): Directory path where the checkpoint will be saved. If not provided, a temporary
+        dirname: Directory path where the checkpoint will be saved. If not provided, a temporary
             directory will be created.
 
     Examples:
-
         .. code-block:: python
 
             from ignite.contrib.handlers.clearml_logger import *
@@ -645,7 +825,7 @@ class ClearMLSaver(DiskSaver):
         dirname: Optional[str] = None,
         *args: Any,
         **kwargs: Any,
-    ) -> None:
+    ):
 
         self._setup_check_clearml(logger, output_uri)
 
@@ -663,7 +843,7 @@ class ClearMLSaver(DiskSaver):
         if "atomic" not in kwargs:
             kwargs["atomic"] = False
 
-        self._checkpoint_slots = defaultdict(list)  # type: DefaultDict[Union[str, Tuple[str, str]], List[Any]]
+        self._checkpoint_slots: DefaultDict[Union[str, Tuple[str, str]], List[Any]] = defaultdict(list)
 
         super(ClearMLSaver, self).__init__(dirname=dirname, *args, **kwargs)  # type: ignore[misc]
 
@@ -676,7 +856,7 @@ class ClearMLSaver(DiskSaver):
                 # Backwards-compatibility for legacy Trains SDK
                 from trains import Task
             except ImportError:
-                raise RuntimeError(
+                raise ModuleNotFoundError(
                     "This contrib module requires clearml to be installed. "
                     "You may install clearml using: \n pip install clearml \n"
                 )
@@ -745,15 +925,13 @@ class ClearMLSaver(DiskSaver):
 
     def __call__(self, checkpoint: Mapping, filename: str, metadata: Optional[Mapping] = None) -> None:
         try:
-            from clearml import Model
             from clearml.binding.frameworks import WeightsFileHandler
         except ImportError:
             try:
                 # Backwards-compatibility for legacy Trains SDK
-                from trains import Model
                 from trains.binding.frameworks import WeightsFileHandler
             except ImportError:
-                raise RuntimeError(
+                raise ModuleNotFoundError(
                     "This contrib module requires clearml to be installed. "
                     "You may install clearml using: \n pip install clearml \n"
                 )
@@ -764,7 +942,7 @@ class ClearMLSaver(DiskSaver):
             warnings.warn("Checkpoint metadata missing or basename cannot be found")
             basename = "checkpoint"
 
-        checkpoint_key = (self.dirname, basename)
+        checkpoint_key = (str(self.dirname), basename)
 
         cb_context = self._CallbacksContext(
             callback_type=WeightsFileHandler.CallbackType,
@@ -793,7 +971,7 @@ class ClearMLSaver(DiskSaver):
             In distributed configuration this method should be called on rank 0 process.
 
         Args:
-            filename (str): artifact name.
+            filename: artifact name.
 
         Returns:
              a local path to a downloaded copy of the artifact
