@@ -1,14 +1,13 @@
-import numbers
+"""Neptune logger and its helper handlers."""
 import tempfile
 import warnings
 from typing import Any, Callable, List, Mapping, Optional, Union
 
 import torch
-import torch.nn as nn
 from torch.optim import Optimizer
 
-import ignite
 import ignite.distributed as idist
+from ignite import __version__
 from ignite.contrib.handlers.base_logger import (
     BaseLogger,
     BaseOptimizerParamsHandler,
@@ -29,70 +28,54 @@ __all__ = [
     "global_step_from_engine",
 ]
 
+_INTEGRATION_VERSION_KEY = "source_code/integrations/neptune-pytorch-ignite"
+
 
 class NeptuneLogger(BaseLogger):
     """
-    `Neptune <https://neptune.ai/>`_ handler to log metrics, model/optimizer parameters, gradients during the training
-    and validation. It can also log model checkpoints to Neptune server.
+    `Neptune <https://neptune.ai/>`_ handler to log metrics, model/optimizer parameters and gradients during training
+    and validation. It can also log model checkpoints to Neptune.
 
     .. code-block:: bash
 
-        pip install neptune-client
+        pip install neptune
 
     Args:
-        api_token (str | None): Required in online mode. Neputne API token, found on https://neptune.ai.
-            Read how to get your API key
-            https://docs.neptune.ai/python-api/tutorials/get-started.html#copy-api-token.
-        project_name (str): Required in online mode. Qualified name of a project in a form of
-           "namespace/project_name" for example "tom/minst-classification".
-           If None, the value of NEPTUNE_PROJECT environment variable will be taken.
-           You need to create the project in https://neptune.ai first.
-        offline_mode (bool): Optional default False. If offline_mode=True no logs will be send to neptune.
-           Usually used for debug purposes.
-        experiment_name (str, optional): Optional. Editable name of the experiment.
-           Name is displayed in the experiment’s Details (Metadata section) and in experiments view as a column.
-        upload_source_files (list, optional): Optional. List of source files to be uploaded.
-           Must be list of str or single str. Uploaded sources are displayed in the experiment’s Source code tab.
-           If None is passed, Python file from which experiment was created will be uploaded.
-           Pass empty list (`[]`) to upload no files. Unix style pathname pattern expansion is supported.
-           For example, you can pass `*.py` to upload all python source files from the current directory.
-           For recursion lookup use `**/*.py` (for Python 3.5 and later). For more information see glob library.
-        params (dict, optional): Optional. Parameters of the experiment. After experiment creation params are read-only.
-           Parameters are displayed in the experiment’s Parameters section and each key-value pair can be
-           viewed in experiments view as a column.
-        properties (dict, optional): Optional default is `{}`. Properties of the experiment.
-           They are editable after experiment is created. Properties are displayed in the experiment’s Details and
-           each key-value pair can be viewed in experiments view as a column.
-        tags (list, optional): Optional default `[]`. Must be list of str. Tags of the experiment.
-           Tags are displayed in the experiment’s Details and can be viewed in experiments view as a column.
+        api_token: Neptune API token, found on https://neptune.ai -> User menu -> "Get your API token".
+           If None, the value of the NEPTUNE_API_TOKEN environment variable is used. To keep your token
+           secure, you should set it to the environment variable rather than including it in your code.
+        project: Name of a Neptune project, in the form "workspace-name/project-name".
+           For example "tom/mnist-classification".
+           If None, the value of the NEPTUNE_PROJECT environment variable is used.
+        **kwargs: Other arguments to be passed to the `init_run()` function.
 
     Examples:
-
         .. code-block:: python
 
             from ignite.contrib.handlers.neptune_logger import *
 
             # Create a logger
-            # We are using the api_token for the anonymous user neptuner but you can use your own.
+            # Note: We are using the API token for anonymous logging. You can pass your own token, or save it as an
+            # environment variable and leave out the api_token argument.
 
             npt_logger = NeptuneLogger(
                 api_token="ANONYMOUS",
-                project_name="shared/pytorch-ignite-integration",
-                experiment_name="cnn-mnist", # Optional,
-                params={"max_epochs": 10}, # Optional,
-                tags=["pytorch-ignite","minst"] # Optional
+                project="common/pytorch-ignite-integration",
+                name="cnn-mnist",  # Optional,
+                tags=["pytorch-ignite", "minst"],  # Optional
             )
 
-            # Attach the logger to the trainer to log training loss at each iteration
+            # Attach the logger to the trainer to log training loss at each iteration.
             npt_logger.attach_output_handler(
                 trainer,
                 event_name=Events.ITERATION_COMPLETED,
                 tag="training",
-                output_transform=lambda loss: {'loss': loss}
+                output_transform=lambda loss: {"loss": loss},
             )
 
-            # Attach the logger to the evaluator on the training dataset and log NLL, Accuracy metrics after each epoch
-            # We setup `global_step_transform=global_step_from_engine(trainer)` to take the epoch
+            # Attach the logger to the evaluator on the training dataset and log NLL
+            # and accuracy metrics after each epoch.
+            # We set up `global_step_transform=global_step_from_engine(trainer)` to take the epoch
             # of the `trainer` instead of `train_evaluator`.
             npt_logger.attach_output_handler(
                 train_evaluator,
@@ -102,43 +85,46 @@ class NeptuneLogger(BaseLogger):
                 global_step_transform=global_step_from_engine(trainer),
             )
 
-            # Attach the logger to the evaluator on the validation dataset and log NLL, Accuracy metrics after
-            # each epoch. We setup `global_step_transform=global_step_from_engine(trainer)` to take the epoch of the
+            # Attach the logger to the evaluator on the validation dataset and log NLL and accuracy metrics after
+            # each epoch. We set up `global_step_transform=global_step_from_engine(trainer)` to take the epoch of the
             # `trainer` instead of `evaluator`.
             npt_logger.attach_output_handler(
                 evaluator,
                 event_name=Events.EPOCH_COMPLETED,
                 tag="validation",
                 metric_names=["nll", "accuracy"],
-                global_step_transform=global_step_from_engine(trainer)),
+                global_step_transform=global_step_from_engine(trainer),
             )
 
-            # Attach the logger to the trainer to log optimizer's parameters, e.g. learning rate at each iteration
+            # Attach the logger to the trainer to log optimizer parameters, such as learning rate at each iteration.
             npt_logger.attach_opt_params_handler(
                 trainer,
                 event_name=Events.ITERATION_STARTED,
                 optimizer=optimizer,
-                param_name='lr'  # optional
+                param_name="lr",  # optional
             )
 
-            # Attach the logger to the trainer to log model's weights norm after each iteration
+            # Attach the logger to the trainer to log model's weights norm after each iteration.
             npt_logger.attach(
                 trainer,
                 event_name=Events.ITERATION_COMPLETED,
-                log_handler=WeightsScalarHandler(model)
+                log_handler=WeightsScalarHandler(model),
             )
 
-        Explore an experiment with neptune tracking here:
-        https://ui.neptune.ai/o/shared/org/pytorch-ignite-integration/e/PYTOR1-18/charts
-        You can save model checkpoints to a Neptune server:
+        Explore runs with Neptune tracking here:
+        https://app.neptune.ai/o/common/org/pytorch-ignite-integration/
+
+        You can also save model checkpoints to a Neptune:
 
         .. code-block:: python
 
             from ignite.handlers import Checkpoint
 
+
             def score_function(engine):
                 return engine.state.metrics["accuracy"]
 
+
             to_save = {"model": model}
             handler = Checkpoint(
                 to_save,
@@ -146,67 +132,65 @@ class NeptuneLogger(BaseLogger):
                 filename_prefix="best",
                 score_function=score_function,
                 score_name="validation_accuracy",
-                global_step_transform=global_step_from_engine(trainer)
+                global_step_transform=global_step_from_engine(trainer),
             )
             validation_evaluator.add_event_handler(Events.COMPLETED, handler)
 
-        It is also possible to use the logger as context manager:
+        It is also possible to use the logger as a context manager:
 
         .. code-block:: python
 
             from ignite.contrib.handlers.neptune_logger import *
 
-            # We are using the api_token for the anonymous user neptuner but you can use your own.
-
-            with NeptuneLogger(api_token="ANONYMOUS",
-                               project_name="shared/pytorch-ignite-integration",
-                               experiment_name="cnn-mnist", # Optional,
-                               params={"max_epochs": 10}, # Optional,
-                               tags=["pytorch-ignite","minst"] # Optional
-                               ) as npt_logger:
-
+            with NeptuneLogger() as npt_logger:
                 trainer = Engine(update_fn)
                 # Attach the logger to the trainer to log training loss at each iteration
                 npt_logger.attach_output_handler(
                     trainer,
                     event_name=Events.ITERATION_COMPLETED,
                     tag="training",
-                    output_transform=lambda loss: {"loss": loss}
+                    output_transform=lambda loss: {"loss": loss},
                 )
 
     """
 
     def __getattr__(self, attr: Any) -> Any:
+        return getattr(self.experiment, attr)
 
-        import neptune
+    def __getitem__(self, key: str) -> Any:
+        return self.experiment[key]
 
-        return getattr(neptune, attr)
+    def __setitem__(self, key: str, val: Any) -> Any:
+        self.experiment[key] = val
 
-    def __init__(self, *args: Any, **kwargs: Any) -> None:
+    def __init__(self, api_token: Optional[str] = None, project: Optional[str] = None, **kwargs: Any) -> None:
         try:
-            import neptune
+            try:
+                # neptune-client<1.0.0 package structure
+                with warnings.catch_warnings():
+                    # ignore the deprecation warnings
+                    warnings.simplefilter("ignore")
+                    import neptune.new as neptune
+            except ImportError:
+                # neptune>=1.0.0 package structure
+                import neptune
         except ImportError:
-            raise RuntimeError(
-                "This contrib module requires neptune-client to be installed. "
-                "You may install neptune with command: \n pip install neptune-client \n"
+            raise ModuleNotFoundError(
+                "This contrib module requires the Neptune client library to be installed. "
+                "Install neptune with the command: \n pip install neptune \n"
             )
 
-        if kwargs.get("offline_mode", False):
-            self.mode = "offline"
-            neptune.init(project_qualified_name="dry-run/project", backend=neptune.OfflineBackend())
-        else:
-            self.mode = "online"
-            neptune.init(api_token=kwargs.get("api_token"), project_qualified_name=kwargs.get("project_name"))
+        run = neptune.init_run(
+            api_token=api_token,
+            project=project,
+            **kwargs,
+        )
+        run[_INTEGRATION_VERSION_KEY] = __version__
 
-        kwargs["name"] = kwargs.pop("experiment_name", None)
-        self._experiment_kwargs = {
-            k: v for k, v in kwargs.items() if k not in ["api_token", "project_name", "offline_mode"]
-        }
-
-        self.experiment = neptune.create_experiment(**self._experiment_kwargs)
+        self.experiment = run
 
     def close(self) -> None:
-        self.stop()
+        self.experiment.stop()
 
     def _create_output_handler(self, *args: Any, **kwargs: Any) -> "OutputHandler":
         return OutputHandler(*args, **kwargs)
@@ -216,10 +200,24 @@ class NeptuneLogger(BaseLogger):
 
 
 class OutputHandler(BaseOutputHandler):
-    """Helper handler to log engine's output and/or metrics
+    """Helper handler to log engine's output and/or metrics.
+
+    Args:
+        tag: common title for all produced plots. For example, "training"
+        metric_names: list of metric names to plot or a string "all" to plot all available
+            metrics.
+        output_transform: output transform function to prepare `engine.state.output` as a number.
+            For example, `output_transform = lambda output: output`
+            This function can also return a dictionary, e.g `{"loss": loss1, "another_loss": loss2}` to label the plot
+            with corresponding keys.
+        global_step_transform: global step transform function to output a desired global step.
+            Input of the function is `(engine, event_name)`. Output of function should be an integer.
+            Default is None, global_step based on attached engine. If provided,
+            uses function output as global_step. To setup global step from another engine, please use
+            :meth:`~ignite.contrib.handlers.neptune_logger.global_step_from_engine`.
+        state_attributes: list of attributes of the ``trainer.state`` to plot.
 
     Examples:
-
         .. code-block:: python
 
             from ignite.contrib.handlers.neptune_logger import *
@@ -292,21 +290,18 @@ class OutputHandler(BaseOutputHandler):
                 global_step_transform=global_step_transform
             )
 
-    Args:
-        tag (str): common title for all produced plots. For example, "training"
-        metric_names (list of str, optional): list of metric names to plot or a string "all" to plot all available
-            metrics.
-        output_transform (callable, optional): output transform function to prepare `engine.state.output` as a number.
-            For example, `output_transform = lambda output: output`
-            This function can also return a dictionary, e.g `{"loss": loss1, "another_loss": loss2}` to label the plot
-            with corresponding keys.
-        global_step_transform (callable, optional): global step transform function to output a desired global step.
-            Input of the function is `(engine, event_name)`. Output of function should be an integer.
-            Default is None, global_step based on attached engine. If provided,
-            uses function output as global_step. To setup global step from another engine, please use
-            :meth:`~ignite.contrib.handlers.neptune_logger.global_step_from_engine`.
+        Another example where the State Attributes ``trainer.state.alpha`` and ``trainer.state.beta``
+        are also logged along with the NLL and Accuracy after each iteration:
 
-    Note:
+        .. code-block:: python
+
+            npt_logger.attach_output_handler(
+                trainer,
+                event_name=Events.ITERATION_COMPLETED,
+                tag="training",
+                metrics=["nll", "accuracy"],
+                state_attributes=["alpha", "beta"],
+            )
 
         Example of `global_step_transform`:
 
@@ -315,6 +310,8 @@ class OutputHandler(BaseOutputHandler):
             def global_step_transform(engine, event_name):
                 return engine.state.get_event_attrib_value(event_name)
 
+    .. versionchanged:: 0.4.7
+        accepts an optional list of `state_attributes`
     """
 
     def __init__(
@@ -322,18 +319,21 @@ class OutputHandler(BaseOutputHandler):
         tag: str,
         metric_names: Optional[Union[str, List[str]]] = None,
         output_transform: Optional[Callable] = None,
-        global_step_transform: Optional[Callable] = None,
-    ) -> None:
-        super(OutputHandler, self).__init__(tag, metric_names, output_transform, global_step_transform)
+        global_step_transform: Optional[Callable[[Engine, Union[str, Events]], int]] = None,
+        state_attributes: Optional[List[str]] = None,
+    ):
+        super(OutputHandler, self).__init__(
+            tag, metric_names, output_transform, global_step_transform, state_attributes
+        )
 
     def __call__(self, engine: Engine, logger: NeptuneLogger, event_name: Union[str, Events]) -> None:
 
         if not isinstance(logger, NeptuneLogger):
             raise TypeError("Handler OutputHandler works only with NeptuneLogger")
 
-        metrics = self._setup_output_metrics(engine)
+        metrics = self._setup_output_metrics_state_attrs(engine, key_tuple=False)
 
-        global_step = self.global_step_transform(engine, event_name)  # type: ignore[misc]
+        global_step = self.global_step_transform(engine, event_name)
 
         if not isinstance(global_step, int):
             raise TypeError(
@@ -342,20 +342,19 @@ class OutputHandler(BaseOutputHandler):
             )
 
         for key, value in metrics.items():
-            if isinstance(value, numbers.Number) or isinstance(value, torch.Tensor) and value.ndimension() == 0:
-                logger.log_metric(f"{self.tag}/{key}", x=global_step, y=value)
-            elif isinstance(value, torch.Tensor) and value.ndimension() == 1:
-                for i, v in enumerate(value):
-                    logger.log_metric(f"{self.tag}/{key}/{i}", x=global_step, y=v.item())
-            else:
-                warnings.warn(f"NeptuneLogger output_handler can not log metrics value type {type(value)}")
+            logger[key].append(value, step=global_step)
 
 
 class OptimizerParamsHandler(BaseOptimizerParamsHandler):
     """Helper handler to log optimizer parameters
 
-    Examples:
+    Args:
+        optimizer: torch optimizer or any object with attribute ``param_groups``
+            as a sequence.
+        param_name: parameter name
+        tag: common title for all produced plots. For example, "generator"
 
+    Examples:
         .. code-block:: python
 
             from ignite.contrib.handlers.neptune_logger import *
@@ -383,15 +382,9 @@ class OptimizerParamsHandler(BaseOptimizerParamsHandler):
                 event_name=Events.ITERATION_STARTED,
                 optimizer=optimizer
             )
-
-    Args:
-        optimizer (torch.optim.Optimizer or object): torch optimizer or any object with attribute ``param_groups``
-            as a sequence.
-        param_name (str): parameter name
-        tag (str, optional): common title for all produced plots. For example, "generator"
     """
 
-    def __init__(self, optimizer: Optimizer, param_name: str = "lr", tag: Optional[str] = None) -> None:
+    def __init__(self, optimizer: Optimizer, param_name: str = "lr", tag: Optional[str] = None):
         super(OptimizerParamsHandler, self).__init__(optimizer, param_name, tag)
 
     def __call__(self, engine: Engine, logger: NeptuneLogger, event_name: Union[str, Events]) -> None:
@@ -406,16 +399,27 @@ class OptimizerParamsHandler(BaseOptimizerParamsHandler):
         }
 
         for k, v in params.items():
-            logger.log_metric(k, x=global_step, y=v)
+            logger[k].append(v, step=global_step)
 
 
 class WeightsScalarHandler(BaseWeightsScalarHandler):
     """Helper handler to log model's weights as scalars.
-    Handler iterates over named parameters of the model, applies reduction function to each parameter
-    produce a scalar and then logs the scalar.
+    Handler, upon construction, iterates over named parameters of the model and keep
+    reference to ones permitted by `whitelist`. Then at every call, applies
+    reduction function to each parameter, produces a scalar and logs it.
+
+    Args:
+        model: model to log weights
+        reduction: function to reduce parameters into scalar
+        tag: common title for all produced plots. For example, "generator"
+        whitelist: specific weights to log. Should be list of model's submodules
+            or parameters names, or a callable which gets weight along with its name
+            and determines if it should be logged. Names should be fully-qualified.
+            For more information please refer to `PyTorch docs
+            <https://pytorch.org/docs/stable/generated/torch.nn.Module.html#torch.nn.Module.get_submodule>`_.
+            If not given, all of model's weights are logged.
 
     Examples:
-
         .. code-block:: python
 
             from ignite.contrib.handlers.neptune_logger import *
@@ -438,16 +442,54 @@ class WeightsScalarHandler(BaseWeightsScalarHandler):
                 log_handler=WeightsScalarHandler(model, reduction=torch.norm)
             )
 
-    Args:
-        model (torch.nn.Module): model to log weights
-        reduction (callable): function to reduce parameters into scalar
-        tag (str, optional): common title for all produced plots. For example, "generator"
+        .. code-block:: python
 
+            from ignite.contrib.handlers.neptune_logger import *
+
+            npt_logger = NeptuneLogger(
+                api_token="ANONYMOUS",
+                project_name="shared/pytorch-ignite-integration",
+                experiment_name="cnn-mnist", # Optional,
+                params={"max_epochs": 10}, # Optional,
+                tags=["pytorch-ignite","minst"] # Optional
+            )
+
+            # Log only `fc` weights
+            npt_logger.attach(
+                trainer,
+                event_name=Events.ITERATION_COMPLETED,
+                log_handler=WeightsScalarHandler(
+                    model,
+                    whitelist=['fc']
+                )
+            )
+
+        .. code-block:: python
+
+            from ignite.contrib.handlers.neptune_logger import *
+
+            npt_logger = NeptuneLogger(
+                api_token="ANONYMOUS",
+                project_name="shared/pytorch-ignite-integration",
+                experiment_name="cnn-mnist", # Optional,
+                params={"max_epochs": 10}, # Optional,
+                tags=["pytorch-ignite","minst"] # Optional
+            )
+
+            # Log weights which have `bias` in their names
+            def has_bias_in_name(n, p):
+                return 'bias' in n
+
+            npt_logger.attach(
+                trainer,
+                event_name=Events.ITERATION_COMPLETED,
+                log_handler=WeightsScalarHandler(model, whitelist=has_bias_in_name)
+            )
+
+    ..  versionchanged:: 0.4.9
+        optional argument `whitelist` added.
     """
 
-    def __init__(self, model: nn.Module, reduction: Callable = torch.norm, tag: Optional[str] = None) -> None:
-        super(WeightsScalarHandler, self).__init__(model, reduction, tag=tag)
-
     def __call__(self, engine: Engine, logger: NeptuneLogger, event_name: Union[str, Events]) -> None:
 
         if not isinstance(logger, NeptuneLogger):
@@ -455,23 +497,33 @@ class WeightsScalarHandler(BaseWeightsScalarHandler):
 
         global_step = engine.state.get_event_attrib_value(event_name)
         tag_prefix = f"{self.tag}/" if self.tag else ""
-        for name, p in self.model.named_parameters():
+        for name, p in self.weights:
             if p.grad is None:
                 continue
 
             name = name.replace(".", "/")
-            logger.log_metric(
-                f"{tag_prefix}weights_{self.reduction.__name__}/{name}", x=global_step, y=self.reduction(p.data),
-            )
+            key = f"{tag_prefix}weights_{self.reduction.__name__}/{name}"
+            logger[key].append(self.reduction(p.data), step=global_step)
 
 
 class GradsScalarHandler(BaseWeightsScalarHandler):
     """Helper handler to log model's gradients as scalars.
-    Handler iterates over the gradients of named parameters of the model, applies reduction function to each parameter
-    produce a scalar and then logs the scalar.
+    Handler, upon construction, iterates over named parameters of the model and keep
+    reference to ones permitted by the `whitelist`. Then at every call, applies
+    reduction function to each parameter's gradient, produces a scalar and logs it.
+
+    Args:
+        model: model to log weights
+        reduction: function to reduce parameters into scalar
+        tag: common title for all produced plots. For example, "generator"
+        whitelist: specific gradients to log. Should be list of model's submodules
+            or parameters names, or a callable which gets weight along with its name
+            and determines if its gradient should be logged. Names should be
+            fully-qualified. For more information please refer to `PyTorch docs
+            <https://pytorch.org/docs/stable/generated/torch.nn.Module.html#torch.nn.Module.get_submodule>`_.
+            If not given, all of model's gradients are logged.
 
     Examples:
-
         .. code-block:: python
 
             from ignite.contrib.handlers.neptune_logger import *
@@ -494,41 +546,82 @@ class GradsScalarHandler(BaseWeightsScalarHandler):
                 log_handler=GradsScalarHandler(model, reduction=torch.norm)
             )
 
-    Args:
-        model (torch.nn.Module): model to log weights
-        reduction (callable): function to reduce parameters into scalar
-        tag (str, optional): common title for all produced plots. For example, "generator"
+        .. code-block:: python
 
+            from ignite.contrib.handlers.neptune_logger import *
+
+            npt_logger = NeptuneLogger(
+                api_token="ANONYMOUS",
+                project_name="shared/pytorch-ignite-integration",
+                experiment_name="cnn-mnist", # Optional,
+                params={"max_epochs": 10}, # Optional,
+                tags=["pytorch-ignite","minst"] # Optional
+            )
+
+            # Log gradient of `base`
+            npt_logger.attach(
+                trainer,
+                event_name=Events.ITERATION_COMPLETED,
+                log_handler=GradsScalarHandler(
+                    model,
+                    reduction=torch.norm,
+                    whitelist=['base']
+                )
+            )
+
+        .. code-block:: python
+
+            from ignite.contrib.handlers.neptune_logger import *
+
+            npt_logger = NeptuneLogger(
+                api_token="ANONYMOUS",
+                project_name="shared/pytorch-ignite-integration",
+                experiment_name="cnn-mnist", # Optional,
+                params={"max_epochs": 10}, # Optional,
+                tags=["pytorch-ignite","minst"] # Optional
+            )
+
+            # Log gradient of weights which belong to a `fc` layer
+            def is_in_fc_layer(n, p):
+                return 'fc' in n
+
+            npt_logger.attach(
+                trainer,
+                event_name=Events.ITERATION_COMPLETED,
+                log_handler=GradsScalarHandler(model, whitelist=is_in_fc_layer)
+            )
+
+    ..  versionchanged:: 0.4.9
+        optional argument `whitelist` added.
     """
 
-    def __init__(self, model: nn.Module, reduction: Callable = torch.norm, tag: Optional[str] = None) -> None:
-        super(GradsScalarHandler, self).__init__(model, reduction, tag=tag)
-
     def __call__(self, engine: Engine, logger: NeptuneLogger, event_name: Union[str, Events]) -> None:
         if not isinstance(logger, NeptuneLogger):
             raise TypeError("Handler GradsScalarHandler works only with NeptuneLogger")
 
         global_step = engine.state.get_event_attrib_value(event_name)
         tag_prefix = f"{self.tag}/" if self.tag else ""
-        for name, p in self.model.named_parameters():
+        for name, p in self.weights:
             if p.grad is None:
                 continue
 
             name = name.replace(".", "/")
-            logger.log_metric(
-                f"{tag_prefix}grads_{self.reduction.__name__}/{name}", x=global_step, y=self.reduction(p.grad),
-            )
+            key = f"{tag_prefix}grads_{self.reduction.__name__}/{name}"
+            logger[key].append(self.reduction(p.grad), step=global_step)
 
 
 class NeptuneSaver(BaseSaveHandler):
     """Handler that saves input checkpoint to the Neptune server.
 
     Args:
-        neptune_logger (ignite.contrib.handlers.neptune_logger.NeptuneLogger): an instance of
+        neptune_logger: an instance of
             NeptuneLogger class.
 
-    Examples:
+    .. Note ::
 
+        NeptuneSaver is currently not supported on Windows.
+
+    Examples:
         .. code-block:: python
 
             from ignite.contrib.handlers.neptune_logger import *
@@ -583,12 +676,30 @@ class NeptuneSaver(BaseSaveHandler):
     def __call__(self, checkpoint: Mapping, filename: str, metadata: Optional[Mapping] = None) -> None:
         # wont work on XLA
 
+        # Imports for BC compatibility
+        try:
+            # neptune-client<1.0.0 package structure
+            with warnings.catch_warnings():
+                # ignore the deprecation warnings
+                warnings.simplefilter("ignore")
+                from neptune.new.types import File
+        except ImportError:
+            # neptune>=1.0.0 package structure
+            from neptune.types import File
+
         with tempfile.NamedTemporaryFile() as tmp:
             # we can not use tmp.name to open tmp.file twice on Win32
             # https://docs.python.org/3/library/tempfile.html#tempfile.NamedTemporaryFile
-            torch.save(checkpoint, tmp.file)  # type: ignore[attr-defined]
-            self._logger.log_artifact(tmp.name, filename)
+            torch.save(checkpoint, tmp.file)
+
+            # rewind the buffer
+            tmp.file.seek(0)
+
+            # hold onto the file stream for uploading.
+            # NOTE: This won't load the whole file in memory and upload
+            #       the stream in smaller chunks.
+            self._logger[filename].upload(File.from_stream(tmp.file))
 
     @idist.one_rank_only(with_barrier=True)
     def remove(self, filename: str) -> None:
-        self._logger.delete_artifacts(filename)
+        del self._logger.experiment[filename]
