@@ -40,6 +40,7 @@ __all__ = [
     "sync",
     "registered_computation_models",
     "one_rank_only",
+    "new_group",
 ]
 
 _model = _SerialModel()
@@ -52,7 +53,7 @@ def sync(temporary: bool = False) -> None:
     This method should be used when distributed context is manually created or destroyed.
 
     Args:
-        temporary (bool): If True, distributed model synchronization is done every call of ``idist.get_*`` methods.
+        temporary: If True, distributed model synchronization is done every call of ``idist.get_*`` methods.
             This may have a negative performance impact.
     """
     global _model
@@ -108,9 +109,8 @@ def backend() -> Optional[str]:
 
 
 def available_backends() -> Tuple[str, ...]:
-    """Returns available backends.
-    """
-    out = ()  # type: Tuple[str, ...]
+    """Returns available backends."""
+    out: Tuple[str, ...] = ()
     for m in registered_computation_models:
         out += m.available_backends
     return out
@@ -134,8 +134,7 @@ def model_name() -> str:
 
 
 def get_world_size() -> int:
-    """Returns world size of current distributed configuration. Returns 1 if no distributed configuration.
-    """
+    """Returns world size of current distributed configuration. Returns 1 if no distributed configuration."""
     if _need_to_sync and isinstance(_model, _SerialModel):
         sync(temporary=True)
 
@@ -143,8 +142,7 @@ def get_world_size() -> int:
 
 
 def get_rank() -> int:
-    """Returns process rank within current distributed configuration. Returns 0 if no distributed configuration.
-    """
+    """Returns process rank within current distributed configuration. Returns 0 if no distributed configuration."""
     if _need_to_sync and isinstance(_model, _SerialModel):
         sync(temporary=True)
 
@@ -152,8 +150,8 @@ def get_rank() -> int:
 
 
 def get_local_rank() -> int:
-    """Returns local process rank within current distributed configuration. Returns 0 if no distributed configuration.
-    """
+    """Returns local process rank within current distributed configuration.
+    Returns 0 if no distributed configuration."""
     if _need_to_sync and isinstance(_model, _SerialModel):
         sync(temporary=True)
 
@@ -191,8 +189,7 @@ def get_node_rank() -> int:
 
 
 def hostname() -> str:
-    """Returns host name for current process within current distributed configuration.
-    """
+    """Returns host name for current process within current distributed configuration."""
     return socket.gethostname()
 
 
@@ -207,8 +204,28 @@ def spawn(
     """Spawns ``nproc_per_node`` processes that run ``fn`` with ``args``/``kwargs_dict`` and initialize
     distributed configuration defined by ``backend``.
 
-    Examples:
+    Args:
+        backend: backend to use: `nccl`, `gloo`, `xla-tpu`, `horovod`
+        fn: function to called as the entrypoint of the spawned process.
+            This function must be defined at the top level of a module so it can be pickled and spawned.
+            This is a requirement imposed by multiprocessing. The function is called as ``fn(i, *args, **kwargs_dict)``,
+            where `i` is the process index and args is the passed through tuple of arguments.
+        args: arguments passed to `fn`.
+        kwargs_dict: kwargs passed to `fn`.
+        nproc_per_node: number of processes to spawn on a single node. Default, 1.
+        kwargs: acceptable kwargs according to provided backend:
 
+            - | "nccl" or "gloo" : ``nnodes`` (default, 1), ``node_rank`` (default, 0), ``master_addr``
+              | (default, "127.0.0.1"), ``master_port`` (default, 2222), ``init_method`` (default, "env://"),
+              | `timeout` to `dist.init_process_group`_ function
+              | and kwargs for `mp.start_processes`_ function.
+
+            - | "xla-tpu" : ``nnodes`` (default, 1), ``node_rank`` (default, 0) and kwargs to `xmp.spawn`_ function.
+
+            - | "horovod": ``hosts`` (default, None) and other kwargs to `hvd_run`_ function. Arguments ``nnodes=1``
+              | and ``node_rank=0`` are tolerated and ignored, otherwise an exception is raised.
+
+    Examples:
         1) Launch single node multi-GPU training using torch native distributed framework
 
         .. code-block:: python
@@ -284,26 +301,6 @@ def spawn(
 
             idist.spawn("xla-tpu", train_fn, args=(a, b, c), kwargs_dict={"d": 23}, nproc_per_node=8)
 
-    Args:
-        backend (str): backend to use: `nccl`, `gloo`, `xla-tpu`, `horovod`
-        fn (function): function to called as the entrypoint of the spawned process.
-            This function must be defined at the top level of a module so it can be pickled and spawned.
-            This is a requirement imposed by multiprocessing. The function is called as ``fn(i, *args, **kwargs_dict)``,
-            where `i` is the process index and args is the passed through tuple of arguments.
-        args (tuple): arguments passed to `fn`.
-        kwargs_dict (Mapping): kwargs passed to `fn`.
-        nproc_per_node (int): number of processes to spawn on a single node. Default, 1.
-        **kwargs: acceptable kwargs according to provided backend:
-
-            - | "nccl" or "gloo" : `nnodes` (default, 1), `node_rank` (default, 0), `master_addr`
-              | (default, "127.0.0.1"), `master_port` (default, 2222), `timeout` to `dist.init_process_group`_ function
-              | and kwargs for `mp.start_processes`_ function.
-
-            - | "xla-tpu" : `nnodes` (default, 1), `node_rank` (default, 0) and kwargs to `xmp.spawn`_ function.
-
-            - | "horovod": `hosts` (default, None) and other kwargs to `hvd_run`_ function. Arguments `nnodes=1`
-              | and `node_rank=0` are tolerated and ignored, otherwise an exception is raised.
-
     .. _dist.init_process_group: https://pytorch.org/docs/stable/distributed.html#torch.distributed.init_process_group
     .. _mp.start_processes: https://pytorch.org/docs/stable/multiprocessing.html#torch.multiprocessing.spawn
     .. _xmp.spawn: http://pytorch.org/xla/release/1.6/index.html#torch_xla.distributed.xla_multiprocessing.spawn
@@ -325,61 +322,85 @@ def spawn(
         )
 
 
-def all_reduce(tensor: Union[torch.Tensor, float], op: str = "SUM") -> Union[torch.Tensor, float]:
+def all_reduce(
+    tensor: Union[torch.Tensor, float], op: str = "SUM", group: Optional[Union[Any, List[int]]] = None
+) -> Union[torch.Tensor, float]:
     """Helper method to perform all reduce operation.
 
     Args:
-        tensor (torch.Tensor or number): tensor or number to collect across participating processes.
-        op (str): reduction operation, "SUM" by default. Possible values: "SUM", "PRODUCT", "MIN", "MAX", "AND", "OR".
-            Please, several values are not supported for the backend like "horovod".
+        tensor: tensor or number to collect across participating processes.
+        op: reduction operation, "SUM" by default. Possible values: "SUM", "PRODUCT", "MIN", "MAX", "AND", "OR".
+            Horovod backend supports only "SUM", "AVERAGE", "ADASUM", "MIN", "MAX", "PRODUCT".
+        group: list of integer or the process group for each backend. If None, the default process group will be used.
 
     Returns:
         torch.Tensor or number
 
+    .. versionchanged:: 0.4.11
+        added ``group``
     """
     if _need_to_sync and isinstance(_model, _SerialModel):
         sync(temporary=True)
 
-    return _model.all_reduce(tensor, op)
+    if isinstance(group, list) and all(isinstance(item, int) for item in group):
+        group = _model.new_group(group)
+
+    return _model.all_reduce(tensor, op, group=group)
 
 
-def all_gather(tensor: Union[torch.Tensor, float, str]) -> Union[torch.Tensor, float, List[float], List[str]]:
+def all_gather(
+    tensor: Union[torch.Tensor, float, str], group: Optional[Union[Any, List[int]]] = None
+) -> Union[torch.Tensor, float, List[float], List[str]]:
     """Helper method to perform all gather operation.
 
     Args:
-        tensor (torch.Tensor or number or str): tensor or number or str to collect across participating processes.
+        tensor: tensor or number or str to collect across participating processes.
+        group: list of integer or the process group for each backend. If None, the default process group will be used.
 
     Returns:
         torch.Tensor of shape ``(world_size * tensor.shape[0], tensor.shape[1], ...)`` if input is a tensor or
         torch.Tensor of shape ``(world_size, )`` if input is a number or
         List of strings if input is a string
 
+    .. versionchanged:: 0.4.11
+        added ``group``
     """
     if _need_to_sync and isinstance(_model, _SerialModel):
         sync(temporary=True)
 
-    return _model.all_gather(tensor)
+    if isinstance(group, list) and all(isinstance(item, int) for item in group):
+        group = _model.new_group(group)
+
+    return _model.all_gather(tensor, group=group)
 
 
-def broadcast(tensor: Union[torch.Tensor, float, str], src: int = 0) -> Union[torch.Tensor, float, str]:
+def broadcast(
+    tensor: Union[torch.Tensor, float, str, None], src: int = 0, safe_mode: bool = False
+) -> Union[torch.Tensor, float, str]:
     """Helper method to perform broadcast operation.
 
     Args:
-        tensor (torch.Tensor or number or str): tensor or number or str to broadcast to participating processes.
-            Make sure to respect dtype of torch tensor input for all processes, otherwise execution will crash.
-        src (int): source rank. Default, 0.
+        tensor: tensor or number or str to broadcast to participating processes.
+            Make sure to respect data type of torch tensor input for all processes, otherwise execution will crash.
+            Can use None for non-source data with ``safe_mode=True``.
+        src: source rank. Default, 0.
+        safe_mode: if True, non source input data can be ``None`` or anything (will be discarded), otherwise data type
+            of the input ``tensor`` should be respected for all processes. Please, keep in mind, this mode is working
+            only for dense tensors as source input if a tensor is provided. It also leads to some additional
+            collectives before the broadcast, making it slower than without using this mode. Default, False.
 
     Returns:
         torch.Tensor or string or number
 
     Examples:
-
         .. code-block:: python
 
+            y = None
             if idist.get_rank() == 0:
                 t1 = torch.rand(4, 5, 6, device=idist.device())
                 s1 = "abc"
                 x = 12.3456
+                y = torch.rand(1, 2, 3, device=idist.device())
             else:
                 t1 = torch.empty(4, 5, 6, device=idist.device())
                 s1 = ""
@@ -397,29 +418,66 @@ def broadcast(tensor: Union[torch.Tensor, float, str], src: int = 0) -> Union[to
             x = idist.broadcast(x, src=0)
             # >>> x = 12.3456
 
+            # Broadcast any of those types from rank 0,
+            # but other ranks do not define the placeholder
+            y = idist.broadcast(y, src=0, safe_mode=True)
+            assert isinstance(y, torch.Tensor)
+
     .. versionadded:: 0.4.2
+
+    .. versionchanged:: 0.4.5
+        added ``safe_mode``
     """
     if _need_to_sync and isinstance(_model, _SerialModel):
         sync(temporary=True)
 
-    return _model.broadcast(tensor, src=src)
+    return _model.broadcast(tensor, src=src, safe_mode=safe_mode)
 
 
 def barrier() -> None:
-    """Helper method to synchronize all processes.
-    """
+    """Helper method to synchronize all processes."""
     if _need_to_sync and isinstance(_model, _SerialModel):
         sync(temporary=True)
 
     _model.barrier()
 
 
+def new_group(ranks: List[int], **kwargs: Any) -> Any:
+    """Helper method to make group for each backend from ranks.
+
+    Args:
+        ranks: subset of ranks to be grouped.
+        kwargs: acceptable kwargs according to provided backend:
+
+            - | "nccl" or "gloo" : ``backend (=None)``, ``pg_options (=None)``.
+
+    Examples:
+        Launch single node multi-GPU training with ``torchrun`` utility.
+
+        .. code-block:: python
+
+            import ignite.distributed as idist
+
+            ranks = [0, 1]
+            group = idist.new_group(ranks)
+
+    .. versionadded:: 0.4.11
+    """
+
+    if _need_to_sync and isinstance(_model, _SerialModel):
+        sync(temporary=True)
+
+    return _model.new_group(ranks, **kwargs)
+
+
 def set_local_rank(index: int) -> None:
     """Method to hint the local rank in case if torch native distributed context is created by user
-    without using :meth:`~ignite.distributed.initialize` or :meth:`~ignite.distributed.spawn`.
+    without using :meth:`~ignite.distributed.utils.initialize` or :meth:`~ignite.distributed.utils.spawn`.
 
-    Usage:
+    Args:
+        index: local rank or current process index
 
+    Examples:
         User set up torch native distributed process group
 
         .. code-block:: python
@@ -432,10 +490,6 @@ def set_local_rank(index: int) -> None:
                 # ...
                 dist.init_process_group(**dist_info)
                 # ...
-
-    Args:
-        index (int): local rank or current process index
-
     """
     from ignite.distributed.comp_models.base import ComputationModel
 
@@ -459,14 +513,22 @@ def _assert_backend(backend: str) -> None:
 def initialize(backend: str, **kwargs: Any) -> None:
     """Initializes distributed configuration according to provided ``backend``
 
-    Examples:
+    Args:
+        backend: backend: `nccl`, `gloo`, `xla-tpu`, `horovod`.
+        kwargs: acceptable kwargs according to provided backend:
 
-        Launch single node multi-GPU training with ``torch.distributed.launch`` utility.
+            - | "nccl" or "gloo" : ``timeout(=timedelta(minutes=30))``, ``init_method(=None)``,
+              | ``rank(=None)``, ``world_size(=None)``.
+              | By default, ``init_method`` will be "env://". See more info about parameters: `torch_init`_.
+
+            - | "horovod" : comm(=None), more info: `hvd_init`_.
+
+    Examples:
+        Launch single node multi-GPU training with ``torchrun`` utility.
 
         .. code-block:: python
 
-            # >>> python -m torch.distributed.launch --nproc_per_node=4 main.py
-
+            # >>> torchrun --nproc_per_node=4 main.py
             # main.py
 
             import ignite.distributed as idist
@@ -480,24 +542,23 @@ def initialize(backend: str, **kwargs: Any) -> None:
                 assert device == torch.device(f"cuda:{local_rank}")
 
 
-            idist.initialize("nccl")
+            backend = "nccl"  # or "gloo" or "horovod" or "xla-tpu"
+            idist.initialize(backend)
+            # or for torch native distributed on Windows:
+            # idist.initialize("nccl", init_method="file://tmp/shared")
             local_rank = idist.get_local_rank()
             train_fn(local_rank, a, b, c)
             idist.finalize()
 
 
-    Args:
-        backend (str, optional): backend: `nccl`, `gloo`, `xla-tpu`, `horovod`.
-        **kwargs: acceptable kwargs according to provided backend:
-
-            - "nccl" or "gloo" : timeout(=timedelta(minutes=30)).
-
-            - "horovod" : comm(=None), more info: `hvd_init`_.
-
-    .. _hvd_init: https://horovod.readthedocs.io/en/latest/api.html#horovod.torch.init
+    .. _torch_init: https://pytorch.org/docs/stable/distributed.html#torch.distributed.init_process_group
+    .. _hvd_init: https://horovod.readthedocs.io/en/latest/api.html#module-horovod.torch
 
     .. versionchanged:: 0.4.2
         ``backend`` now accepts `horovod` distributed framework.
+
+    .. versionchanged:: 0.4.5
+        ``kwargs`` now accepts ``init_method``, ``rank``, ``world_size`` for PyTorch native distributed backend.
     """
     if not (has_xla_support or has_native_dist_support or has_hvd_support):
         # nothing to do => serial model
@@ -521,8 +582,7 @@ def finalize() -> None:
 
 
 def show_config() -> None:
-    """Helper method to display distributed configuration via ``logging``.
-    """
+    """Helper method to display distributed configuration via ``logging``."""
 
     # setup parallel logger
     logger = setup_logger(__name__)
@@ -543,22 +603,23 @@ def one_rank_only(rank: int = 0, with_barrier: bool = False) -> Callable:
     """Decorator to filter handlers wrt a rank number
 
     Args:
-        rank (int): rank number of the handler (default: 0).
-        with_barrier (bool): synchronisation with a barrier (default: False).
+        rank: rank number of the handler (default: 0).
+        with_barrier: synchronisation with a barrier (default: False).
 
-    .. code-block:: python
+    Examples:
+        .. code-block:: python
 
-        engine = ...
+            engine = ...
 
-        @engine.on(...)
-        @one_rank_only() # means @one_rank_only(rank=0)
-        def some_handler(_):
-            ...
+            @engine.on(...)
+            @one_rank_only() # means @one_rank_only(rank=0)
+            def some_handler(_):
+                ...
 
-        @engine.on(...)
-        @one_rank_only(rank=1)
-        def some_handler(_):
-            ...
+            @engine.on(...)
+            @one_rank_only(rank=1)
+            def some_handler(_):
+                ...
     """
 
     def _one_rank_only(func: Callable) -> Callable:
