@@ -1,9 +1,10 @@
 import numbers
 import warnings
 import weakref
+from collections.abc import Sequence
 from enum import Enum
 from types import DynamicClassAttribute
-from typing import TYPE_CHECKING, Any, Callable, Dict, Iterable, Iterator, List, Optional, Union
+from typing import Any, Callable, Dict, Iterable, Iterator, List, Optional, TYPE_CHECKING, Union
 
 from torch.utils.data import DataLoader
 
@@ -20,17 +21,14 @@ class CallableEventWithFilter:
     be run at the current event (if the event type is correct)
 
     Args:
-        value (str): The actual enum value. Only needed for internal use. Do not touch!
-        event_filter (callable): A function taking the engine and the current event value as input and returning a
+        value: The actual enum value. Only needed for internal use. Do not touch!
+        event_filter: A function taking the engine and the current event value as input and returning a
             boolean to indicate whether this event should be executed. Defaults to None, which will result to a
             function that always returns `True`
-        name (str, optional): The enum-name of the current object. Only needed for internal use. Do not touch!
-
+        name: The enum-name of the current object. Only needed for internal use. Do not touch!
     """
 
     def __init__(self, value: str, event_filter: Optional[Callable] = None, name: Optional[str] = None) -> None:
-        if event_filter is None:
-            event_filter = CallableEventWithFilter.default_event_filter
         self.filter = event_filter
 
         if not hasattr(self, "_value_"):
@@ -51,24 +49,40 @@ class CallableEventWithFilter:
         return self._value_
 
     def __call__(
-        self, event_filter: Optional[Callable] = None, every: Optional[int] = None, once: Optional[int] = None
+        self,
+        event_filter: Optional[Callable] = None,
+        every: Optional[int] = None,
+        once: Optional[Union[int, List]] = None,
+        before: Optional[int] = None,
+        after: Optional[int] = None,
     ) -> "CallableEventWithFilter":
         """
         Makes the event class callable and accepts either an arbitrary callable as filter
         (which must take in the engine and current event value and return a boolean) or an every or once value
 
         Args:
-            event_filter (callable, optional): a filter function to check if the event should be executed when
+            event_filter: a filter function to check if the event should be executed when
                 the event type was fired
-            every (int, optional): a value specifying how often the event should be fired
-            once (int, optional): a value specifying when the event should be fired (if only once)
+            every: a value specifying how often the event should be fired
+            once: a value or list of values specifying when the event should be fired (if only once)
+            before: a value specifying the number of occurrence that event should be fired before
+            after: a value specifying the number of occurrence that event should be fired after
 
         Returns:
             CallableEventWithFilter: A new event having the same value but a different filter function
         """
 
-        if not ((event_filter is not None) ^ (every is not None) ^ (once is not None)):
-            raise ValueError("Only one of the input arguments should be specified")
+        if (
+            sum(
+                (
+                    event_filter is not None,
+                    once is not None,
+                    (every is not None or before is not None or after is not None),
+                )
+            )
+            != 1
+        ):
+            raise ValueError("Only one of the input arguments should be specified, except before, after and every")
 
         if (event_filter is not None) and not callable(event_filter):
             raise TypeError("Argument event_filter should be a callable")
@@ -76,8 +90,19 @@ class CallableEventWithFilter:
         if (every is not None) and not (isinstance(every, numbers.Integral) and every > 0):
             raise ValueError("Argument every should be integer and greater than zero")
 
-        if (once is not None) and not (isinstance(once, numbers.Integral) and once > 0):
-            raise ValueError("Argument every should be integer and positive")
+        if once is not None:
+            c1 = isinstance(once, numbers.Integral) and once > 0
+            c2 = isinstance(once, Sequence) and len(once) > 0 and all(isinstance(e, int) and e > 0 for e in once)
+            if not (c1 or c2):
+                raise ValueError(
+                    f"Argument once should either be a positive integer or a list of positive integers, got {once}"
+                )
+
+        if (before is not None) and not (isinstance(before, numbers.Integral) and before >= 0):
+            raise ValueError("Argument before should be integer and greater or equal to zero")
+
+        if (after is not None) and not (isinstance(after, numbers.Integral) and after >= 0):
+            raise ValueError("Argument after should be integer and greater or equal to zero")
 
         if every is not None:
             if every == 1:
@@ -87,7 +112,13 @@ class CallableEventWithFilter:
                 event_filter = self.every_event_filter(every)
 
         if once is not None:
-            event_filter = self.once_event_filter(once)
+            event_filter = self.once_event_filter([once] if isinstance(once, int) else once)
+
+        if before is not None or after is not None:
+            if every is not None:
+                event_filter = self.every_before_and_after_event_filter(every, before, after)
+            else:
+                event_filter = self.before_and_after_event_filter(before, after)
 
         # check signature:
         if event_filter is not None:
@@ -97,6 +128,8 @@ class CallableEventWithFilter:
 
     @staticmethod
     def every_event_filter(every: int) -> Callable:
+        """A wrapper for every event filter."""
+
         def wrapper(engine: "Engine", event: int) -> bool:
             if event % every == 0:
                 return True
@@ -105,9 +138,39 @@ class CallableEventWithFilter:
         return wrapper
 
     @staticmethod
-    def once_event_filter(once: int) -> Callable:
+    def once_event_filter(once: List) -> Callable:
+        """A wrapper for once event filter."""
+
         def wrapper(engine: "Engine", event: int) -> bool:
-            if event == once:
+            if event in once:
+                return True
+            return False
+
+        return wrapper
+
+    @staticmethod
+    def before_and_after_event_filter(before: Optional[int] = None, after: Optional[int] = None) -> Callable:
+        """A wrapper for before and after event filter."""
+        before_: Union[int, float] = float("inf") if before is None else before
+        after_: int = 0 if after is None else after
+
+        def wrapper(engine: "Engine", event: int) -> bool:
+            if event > after_ and event < before_:
+                return True
+            return False
+
+        return wrapper
+
+    @staticmethod
+    def every_before_and_after_event_filter(
+        every: int, before: Optional[int] = None, after: Optional[int] = None
+    ) -> Callable:
+        """A wrapper which triggers for every `every` iterations after `after` and before `before`."""
+        before_: Union[int, float] = float("inf") if before is None else before
+        after_: int = 0 if after is None else after
+
+        def wrapper(engine: "Engine", event: int) -> bool:
+            if after_ < event < before_ and (event - after_ - 1) % every == 0:
                 return True
             return False
 
@@ -115,10 +178,15 @@ class CallableEventWithFilter:
 
     @staticmethod
     def default_event_filter(engine: "Engine", event: int) -> bool:
+        """Default event filter. This method is is deprecated and will be removed. Please, use None instead"""
+        warnings.warn("Events.default_event_filter is deprecated and will be removed. Please, use None instead")
         return True
 
-    def __str__(self) -> str:
-        return "<event=%s, filter=%r>" % (self.name, self.filter)
+    def __repr__(self) -> str:
+        out = f"Events.{self.name}"
+        if self.filter is not None:
+            out += f"(filter={self.filter})"
+        return out
 
     def __eq__(self, other: Any) -> bool:
         if isinstance(other, CallableEventWithFilter):
@@ -146,9 +214,12 @@ class CallableEvents(CallableEventWithFilter):
         )
 
 
-class EventEnum(CallableEventWithFilter, Enum):  # type: ignore[misc]
+class EventEnum(CallableEventWithFilter, Enum):
     """Base class for all :class:`~ignite.engine.events.Events`. User defined custom events should also inherit
-    this class. For example, Custom events based on the loss calculation and backward pass can be created as follows:
+    this class.
+
+     Examples:
+         Custom events based on the loss calculation and backward pass can be created as follows:
 
         .. code-block:: python
 
@@ -177,7 +248,10 @@ class EventEnum(CallableEventWithFilter, Enum):  # type: ignore[misc]
                 # ...
     """
 
-    pass
+    def __new__(cls, value: str) -> "EventEnum":
+        obj = CallableEventWithFilter.__new__(cls)
+        obj._value_ = value
+        return obj
 
 
 class Events(EventEnum):
@@ -253,6 +327,16 @@ class Events(EventEnum):
         def call_once(engine):
             # do something on 50th iteration
 
+        # d) "before" and "after" event filter
+        @engine.on(Events.EPOCH_STARTED(before=30, after=10))
+        def call_before(engine):
+            # do something in 11 to 29 epoch
+
+        # e) Mixing "every" and "before" / "after" event filters
+        @engine.on(Events.EPOCH_STARTED(every=5, before=25, after=8))
+        def call_every_itr_before_after(engine):
+            # do something on 9, 14, 19, 24 epochs
+
     Event filter function `event_filter` accepts as input `engine` and `event` and should return True/False.
     Argument `event` is the value of iteration or epoch, depending on which type of Events the function is passed.
 
@@ -277,21 +361,36 @@ class Events(EventEnum):
     """
 
     EPOCH_STARTED = "epoch_started"
+    """triggered when the epoch is started."""
     EPOCH_COMPLETED = "epoch_completed"
+    """Event attribute indicating epoch is ended."""
 
     STARTED = "started"
+    """triggered when engine's run is started."""
     COMPLETED = "completed"
+    """triggered when engine's run is completed"""
 
     ITERATION_STARTED = "iteration_started"
+    """triggered when an iteration is started."""
     ITERATION_COMPLETED = "iteration_completed"
+    """triggered when the iteration is ended."""
     EXCEPTION_RAISED = "exception_raised"
+    """triggered when an exception is encountered."""
 
     GET_BATCH_STARTED = "get_batch_started"
+    """triggered before next batch is fetched."""
     GET_BATCH_COMPLETED = "get_batch_completed"
+    """triggered after the batch is fetched."""
 
     DATALOADER_STOP_ITERATION = "dataloader_stop_iteration"
+    """engine's specific event triggered when dataloader has no more data to provide"""
     TERMINATE = "terminate"
+    """triggered when the run is about to end completely, after receiving terminate() call."""
     TERMINATE_SINGLE_EPOCH = "terminate_single_epoch"
+    """triggered when the run is about to end the current epoch,
+    after receiving a terminate_epoch() call."""
+    INTERRUPT = "interrupt"
+    """triggered when the run is interrupted, after receiving interrupt() call."""
 
     def __or__(self, other: Any) -> "EventsList":
         return EventsList() | self | other
@@ -313,7 +412,7 @@ class EventsList:
 
     or
 
-     .. code-block:: python
+    .. code-block:: python
 
         @engine.on(Events.STARTED | Events.COMPLETED | Events.ITERATION_STARTED(every=3))
         def call_on_events(engine):
@@ -322,7 +421,7 @@ class EventsList:
     """
 
     def __init__(self) -> None:
-        self._events = []  # type: List[Union[Events, CallableEventWithFilter]]
+        self._events: List[Union[Events, CallableEventWithFilter]] = []
 
     def _append(self, event: Union[Events, CallableEventWithFilter]) -> None:
         if not isinstance(event, (Events, CallableEventWithFilter)):
@@ -361,9 +460,11 @@ class State:
         state.times             # dictionary with total and per-epoch times fetched on
                                 # keys: Events.EPOCH_COMPLETED.name and Events.COMPLETED.name
 
+    Args:
+        kwargs: keyword arguments to be defined as State attributes.
     """
 
-    event_to_attr = {
+    event_to_attr: Dict[Union[str, "Events", "CallableEventWithFilter"], str] = {
         Events.GET_BATCH_STARTED: "iteration",
         Events.GET_BATCH_COMPLETED: "iteration",
         Events.ITERATION_STARTED: "iteration",
@@ -372,22 +473,22 @@ class State:
         Events.EPOCH_COMPLETED: "epoch",
         Events.STARTED: "epoch",
         Events.COMPLETED: "epoch",
-    }  # type: Dict[Union[str, "Events", "CallableEventWithFilter"], str]
+    }
 
     def __init__(self, **kwargs: Any) -> None:
         self.iteration = 0
         self.epoch = 0
-        self.epoch_length = None  # type: Optional[int]
-        self.max_epochs = None  # type: Optional[int]
-        self.output = None  # type: Optional[int]
-        self.batch = None  # type: Optional[int]
-        self.metrics = {}  # type: Dict[str, Any]
-        self.dataloader = None  # type: Optional[Union[DataLoader, Iterable[Any]]]
-        self.seed = None  # type: Optional[int]
-        self.times = {
+        self.epoch_length: Optional[int] = None
+        self.max_epochs: Optional[int] = None
+        self.output: Optional[int] = None
+        self.batch: Optional[int] = None
+        self.metrics: Dict[str, Any] = {}
+        self.dataloader: Optional[Union[DataLoader, Iterable[Any]]] = None
+        self.seed: Optional[int] = None
+        self.times: Dict[str, Optional[float]] = {
             Events.EPOCH_COMPLETED.name: None,
             Events.COMPLETED.name: None,
-        }  # type: Dict[str, Optional[float]]
+        }
 
         for k, v in kwargs.items():
             setattr(self, k, v)
@@ -400,6 +501,7 @@ class State:
                 setattr(self, value, 0)
 
     def get_event_attrib_value(self, event_name: Union[str, Events, CallableEventWithFilter]) -> int:
+        """Get the value of Event attribute with given `event_name`."""
         if event_name not in State.event_to_attr:
             raise RuntimeError(f"Unknown event name '{event_name}'")
         return getattr(self, State.event_to_attr[event_name])
@@ -426,20 +528,19 @@ class RemovableEventHandle:
         handler: Registered event handler, stored as weakref.
         engine: Target engine, stored as weakref.
 
-    Example usage:
+    Examples:
+        .. code-block:: python
 
-    .. code-block:: python
+            engine = Engine()
 
-        engine = Engine()
+            def print_epoch(engine):
+                print(f"Epoch: {engine.state.epoch}")
 
-        def print_epoch(engine):
-            print(f"Epoch: {engine.state.epoch}")
+            with engine.add_event_handler(Events.EPOCH_COMPLETED, print_epoch):
+                # print_epoch handler registered for a single run
+                engine.run(data)
 
-        with engine.add_event_handler(Events.EPOCH_COMPLETED, print_epoch):
-            # print_epoch handler registered for a single run
-            engine.run(data)
-
-        # print_epoch handler is now unregistered
+            # print_epoch handler is now unregistered
     """
 
     def __init__(
@@ -457,6 +558,14 @@ class RemovableEventHandle:
         if handler is None or engine is None:
             return
 
+        if hasattr(handler, "_parent"):
+            handler = handler._parent()
+            if handler is None:
+                raise RuntimeError(
+                    "Internal error! Please fill an issue on https://github.com/pytorch/ignite/issues "
+                    "if encounter this error. Thank you!"
+                )
+
         if isinstance(self.event_name, EventsList):
             for e in self.event_name:
                 if engine.has_event_handler(handler, e):
