@@ -1,8 +1,8 @@
-import warnings
 from abc import ABCMeta, abstractmethod
 from collections.abc import Mapping
 from functools import wraps
-from typing import TYPE_CHECKING, Any, Callable, Dict, Optional, Tuple, Union
+from numbers import Number
+from typing import Any, Callable, cast, Dict, Optional, Sequence, Tuple, TYPE_CHECKING, Union
 
 import torch
 
@@ -24,11 +24,11 @@ class MetricUsage:
 
     Args:
         started: event when the metric starts to compute. This event will be associated to
-            :meth:`~ignite.metrics.Metric.started`.
+            :meth:`~ignite.metrics.metric.Metric.started`.
         completed: event when the metric completes. This event will be associated to
-            :meth:`~ignite.metrics.Metric.completed`.
+            :meth:`~ignite.metrics.metric.Metric.completed`.
         iteration_completed: event when the metric updates. This event will be associated to
-            :meth:`~ignite.metrics.Metric.iteration_completed`.
+            :meth:`~ignite.metrics.metric.Metric.iteration_completed`.
     """
 
     def __init__(self, started: Events, completed: Events, iteration_completed: CallableEventWithFilter) -> None:
@@ -55,15 +55,16 @@ class EpochWise(MetricUsage):
 
     Metric's methods are triggered on the following engine events:
 
-    - :meth:`~ignite.metrics.Metric.started` on every ``EPOCH_STARTED`` (See :class:`~ignite.engine.events.Events`).
-    - :meth:`~ignite.metrics.Metric.iteration_completed` on every ``ITERATION_COMPLETED``.
-    - :meth:`~ignite.metrics.Metric.completed` on every ``EPOCH_COMPLETED``.
+    - :meth:`~ignite.metrics.metric.Metric.started` on every ``EPOCH_STARTED``
+      (See :class:`~ignite.engine.events.Events`).
+    - :meth:`~ignite.metrics.metric.Metric.iteration_completed` on every ``ITERATION_COMPLETED``.
+    - :meth:`~ignite.metrics.metric.Metric.completed` on every ``EPOCH_COMPLETED``.
 
     Attributes:
-        usage_name (str): usage name string
+        usage_name: usage name string
     """
 
-    usage_name = "epoch_wise"
+    usage_name: str = "epoch_wise"
 
     def __init__(self) -> None:
         super(EpochWise, self).__init__(
@@ -79,15 +80,16 @@ class BatchWise(MetricUsage):
 
     Metric's methods are triggered on the following engine events:
 
-    - :meth:`~ignite.metrics.Metric.started` on every ``ITERATION_STARTED`` (See :class:`~ignite.engine.events.Events`).
-    - :meth:`~ignite.metrics.Metric.iteration_completed` on every ``ITERATION_COMPLETED``.
-    - :meth:`~ignite.metrics.Metric.completed` on every ``ITERATION_COMPLETED``.
+    - :meth:`~ignite.metrics.metric.Metric.started` on every ``ITERATION_STARTED``
+      (See :class:`~ignite.engine.events.Events`).
+    - :meth:`~ignite.metrics.metric.Metric.iteration_completed` on every ``ITERATION_COMPLETED``.
+    - :meth:`~ignite.metrics.metric.Metric.completed` on every ``ITERATION_COMPLETED``.
 
     Attributes:
-        usage_name (str): usage name string
+        usage_name: usage name string
     """
 
-    usage_name = "batch_wise"
+    usage_name: str = "batch_wise"
 
     def __init__(self) -> None:
         super(BatchWise, self).__init__(
@@ -103,14 +105,15 @@ class BatchFiltered(MetricUsage):
 
     Metric's methods are triggered on the following engine events:
 
-    - :meth:`~ignite.metrics.Metric.started` on every ``EPOCH_STARTED`` (See :class:`~ignite.engine.events.Events`).
-    - :meth:`~ignite.metrics.Metric.iteration_completed` on filtered ``ITERATION_COMPLETED``.
-    - :meth:`~ignite.metrics.Metric.completed` on every ``EPOCH_COMPLETED``.
+    - :meth:`~ignite.metrics.metric.Metric.started` on every ``EPOCH_STARTED``
+      (See :class:`~ignite.engine.events.Events`).
+    - :meth:`~ignite.metrics.metric.Metric.iteration_completed` on filtered ``ITERATION_COMPLETED``.
+    - :meth:`~ignite.metrics.metric.Metric.completed` on every ``EPOCH_COMPLETED``.
 
     Args:
-        *args: Positional arguments to setup :attr:`~ignite.engine.events.Events.ITERATION_COMPLETED(*args, **kwargs)`
-        **kwargs: Keyword arguments to setup :attr:`~ignite.engine.events.Events.ITERATION_COMPLETED(*args, **kwargs)`
-            handled by :meth:`~ignite.metrics.Metric.iteration_completed`.
+        args: Positional arguments to setup :attr:`~ignite.engine.events.Events.ITERATION_COMPLETED`
+        kwargs: Keyword arguments to setup :attr:`~ignite.engine.events.Events.ITERATION_COMPLETED`
+            handled by :meth:`~ignite.metrics.metric.Metric.iteration_completed`.
 
     """
 
@@ -127,26 +130,27 @@ class Metric(metaclass=ABCMeta):
     Base class for all Metrics.
 
     Args:
-        output_transform (callable, optional): a callable that is used to transform the
+        output_transform: a callable that is used to transform the
             :class:`~ignite.engine.engine.Engine`'s ``process_function``'s output into the
             form expected by the metric. This can be useful if, for example, you have a multi-output model and
             you want to compute the metric with respect to one of the outputs.
             By default, metrics require the output as ``(y_pred, y)`` or ``{'y_pred': y_pred, 'y': y}``.
-        device (str or torch.device): specifies which device updates are accumulated on. Setting the
+        device: specifies which device updates are accumulated on. Setting the
             metric's device to be the same as your ``update`` arguments ensures the ``update`` method is
             non-blocking. By default, CPU.
 
     Attributes:
-        required_output_keys (tuple): dictionary defines required keys to be found in ``engine.state.output`` if the
-            latter is a dictionary. Default, ``("y_pred", "y")``. This is useful with custom metrics that can require
-            other arguments than predictions ``y_pred`` and targets ``y``. See notes below for an example.
-
-    Note:
+        required_output_keys: dictionary defines required keys to be found in ``engine.state.output`` if the
+            latter is a dictionary. By default, ``("y_pred", "y")``. This is useful with custom metrics that can require
+            other arguments than predictions ``y_pred`` and targets ``y``. See an example below.
 
+    Examples:
         Let's implement a custom metric that requires ``y_pred``, ``y`` and ``x`` as input for ``update`` function.
         In the example below we show how to setup standard metric like Accuracy and the custom metric using by an
         ``evaluator`` created with :meth:`~ignite.engine.create_supervised_evaluator` method.
 
+        For more information on how metric works with :class:`~ignite.engine.engine.Engine`, visit :ref:`attach-engine`.
+
         .. code-block:: python
 
             # https://discuss.pytorch.org/t/how-access-inputs-in-custom-ignite-metric/91221/5
@@ -196,32 +200,20 @@ class Metric(metaclass=ABCMeta):
     """
 
     # public class attribute
-    required_output_keys = ("y_pred", "y")  # type: Optional[Tuple]
+    required_output_keys: Optional[Tuple] = ("y_pred", "y")
     # for backward compatibility
     _required_output_keys = required_output_keys
 
     def __init__(
-        self, output_transform: Callable = lambda x: x, device: Union[str, torch.device] = torch.device("cpu"),
+        self, output_transform: Callable = lambda x: x, device: Union[str, torch.device] = torch.device("cpu")
     ):
         self._output_transform = output_transform
 
-        # Check device if distributed is initialized:
-        if idist.get_world_size() > 1:
-
-            # check if reset and update methods are decorated. Compute may not be decorated
-            if not (hasattr(self.reset, "_decorated") and hasattr(self.update, "_decorated")):
-                warnings.warn(
-                    f"{self.__class__.__name__} class does not support distributed setting. "
-                    "Computed result is not collected across all computing devices",
-                    RuntimeWarning,
-                )
-
         # Some metrics have a large performance regression when run on XLA devices, so for now, we disallow it.
         if torch.device(device).type == "xla":
             raise ValueError("Cannot create metric on an XLA device. Use device='cpu' instead.")
 
         self._device = torch.device(device)
-        self._is_reduced = False
         self.reset()
 
     @abstractmethod
@@ -255,7 +247,7 @@ class Metric(metaclass=ABCMeta):
         Returns:
             Any: | the actual quantity of interest. However, if a :class:`~collections.abc.Mapping` is returned,
                  it will be (shallow) flattened into `engine.state.metrics` when
-                 :func:`~ignite.metrics.Metric.completed` is called.
+                 :func:`~ignite.metrics.metric.Metric.completed` is called.
 
         Raises:
             NotComputableError: raised when the metric cannot be computed.
@@ -264,20 +256,29 @@ class Metric(metaclass=ABCMeta):
 
     def started(self, engine: Engine) -> None:
         """Helper method to start data gathering for metric's computation. It is automatically attached to the
-        `engine` with :meth:`~ignite.metrics.Metric.attach`.
+        `engine` with :meth:`~ignite.metrics.metric.Metric.attach`.
 
         Args:
-            engine (Engine): the engine to which the metric must be attached
+            engine: the engine to which the metric must be attached
         """
         self.reset()
 
     @torch.no_grad()
     def iteration_completed(self, engine: Engine) -> None:
         """Helper method to update metric's computation. It is automatically attached to the
-        `engine` with :meth:`~ignite.metrics.Metric.attach`.
+        `engine` with :meth:`~ignite.metrics.metric.Metric.attach`.
 
         Args:
-            engine (Engine): the engine to which the metric must be attached
+            engine: the engine to which the metric must be attached
+
+        Note:
+            ``engine.state.output`` is used to compute metric values.
+            The majority of implemented metrics accepts the following formats for ``engine.state.output``:
+            ``(y_pred, y)`` or ``{'y_pred': y_pred, 'y': y}``. ``y_pred`` and ``y`` can be torch tensors or
+            list of tensors/numbers if applicable.
+
+        .. versionchanged:: 0.4.5
+            ``y_pred`` and ``y`` can be torch tensors or list of tensors/numbers
         """
 
         output = self._output_transform(engine.state.output)
@@ -293,18 +294,36 @@ class Metric(metaclass=ABCMeta):
                     f"it should contain {self.required_output_keys} keys, but given {list(output.keys())}"
                 )
             output = tuple(output[k] for k in self.required_output_keys)
-        self.update(output)
+
+        if isinstance(output, Sequence) and all([_is_list_of_tensors_or_numbers(o) for o in output]):
+            if not (len(output) == 2 and len(output[0]) == len(output[1])):
+                raise ValueError(
+                    f"Output should have 2 items of the same length, "
+                    f"got {len(output)} and {len(output[0])}, {len(output[1])}"
+                )
+            for o1, o2 in zip(output[0], output[1]):
+                # o1 and o2 are list of tensors or numbers
+                tensor_o1 = _to_batched_tensor(o1)
+                tensor_o2 = _to_batched_tensor(o2, device=tensor_o1.device)
+                self.update((tensor_o1, tensor_o2))
+        else:
+            self.update(output)
 
     def completed(self, engine: Engine, name: str) -> None:
         """Helper method to compute metric's value and put into the engine. It is automatically attached to the
-        `engine` with :meth:`~ignite.metrics.Metric.attach`.
+        `engine` with :meth:`~ignite.metrics.metric.Metric.attach`. If metrics' value is torch tensor, it is
+        explicitly sent to CPU device.
 
         Args:
-            engine (Engine): the engine to which the metric must be attached
-            name (str): the name of the metric used as key in dict `engine.state.metrics`
+            engine: the engine to which the metric must be attached
+            name: the name of the metric used as key in dict `engine.state.metrics`
 
         .. versionchanged:: 0.4.3
             Added dict in metrics results.
+
+        .. versionchanged:: 0.4.5
+            metric's value is put on CPU if torch tensor.
+
         """
         result = self.compute()
         if isinstance(result, Mapping):
@@ -315,8 +334,11 @@ class Metric(metaclass=ABCMeta):
                 engine.state.metrics[key] = value
             engine.state.metrics[name] = result
         else:
-            if isinstance(result, torch.Tensor) and len(result.size()) == 0:
-                result = result.item()
+            if isinstance(result, torch.Tensor):
+                if len(result.size()) == 0:
+                    result = result.item()
+                elif "cpu" not in result.device.type:
+                    result = result.cpu()
 
             engine.state.metrics[name] = result
 
@@ -338,33 +360,33 @@ class Metric(metaclass=ABCMeta):
         contain computed metric's value under provided name.
 
         Args:
-            engine (Engine): the engine to which the metric must be attached
-            name (str): the name of the metric to attach
-            usage (str or MetricUsage, optional): the usage of the metric. Valid string values should be
-                :attr:`ignite.metrics.EpochWise.usage_name` (default) or
-                :attr:`ignite.metrics.BatchWise.usage_name`.
+            engine: the engine to which the metric must be attached
+            name: the name of the metric to attach
+            usage: the usage of the metric. Valid string values should be
+                :attr:`ignite.metrics.metric.EpochWise.usage_name` (default) or
+                :attr:`ignite.metrics.metric.BatchWise.usage_name`.
 
-        Example:
+        Examples:
 
-        .. code-block:: python
+            .. code-block:: python
 
-            metric = ...
-            metric.attach(engine, "mymetric")
+                metric = ...
+                metric.attach(engine, "mymetric")
 
-            assert "mymetric" in engine.run(data).metrics
+                assert "mymetric" in engine.run(data).metrics
 
-            assert metric.is_attached(engine)
+                assert metric.is_attached(engine)
 
-        Example with usage:
+            Example with usage:
 
-        .. code-block:: python
+            .. code-block:: python
 
-            metric = ...
-            metric.attach(engine, "mymetric", usage=BatchWise.usage_name)
+                metric = ...
+                metric.attach(engine, "mymetric", usage=BatchWise.usage_name)
 
-            assert "mymetric" in engine.run(data).metrics
+                assert "mymetric" in engine.run(data).metrics
 
-            assert metric.is_attached(engine, usage=BatchWise.usage_name)
+                assert metric.is_attached(engine, usage=BatchWise.usage_name)
         """
         usage = self._check_usage(usage)
         if not engine.has_event_handler(self.started, usage.STARTED):
@@ -376,38 +398,37 @@ class Metric(metaclass=ABCMeta):
     def detach(self, engine: Engine, usage: Union[str, MetricUsage] = EpochWise()) -> None:
         """
         Detaches current metric from the engine and no metric's computation is done during the run.
-        This method in conjunction with :meth:`~ignite.metrics.Metric.attach` can be useful if several
+        This method in conjunction with :meth:`~ignite.metrics.metric.Metric.attach` can be useful if several
         metrics need to be computed with different periods. For example, one metric is computed every training epoch
         and another metric (e.g. more expensive one) is done every n-th training epoch.
 
         Args:
-            engine (Engine): the engine from which the metric must be detached
-            usage (str or MetricUsage, optional): the usage of the metric. Valid string values should be
+            engine: the engine from which the metric must be detached
+            usage: the usage of the metric. Valid string values should be
                 'epoch_wise' (default) or 'batch_wise'.
 
-        Example:
+        Examples:
+            .. code-block:: python
 
-        .. code-block:: python
+                metric = ...
+                engine = ...
+                metric.detach(engine)
 
-            metric = ...
-            engine = ...
-            metric.detach(engine)
+                assert "mymetric" not in engine.run(data).metrics
 
-            assert "mymetric" not in engine.run(data).metrics
+                assert not metric.is_attached(engine)
 
-            assert not metric.is_attached(engine)
+            Example with usage:
 
-        Example with usage:
+            .. code-block:: python
 
-        .. code-block:: python
+                metric = ...
+                engine = ...
+                metric.detach(engine, usage="batch_wise")
 
-            metric = ...
-            engine = ...
-            metric.detach(engine, usage="batch_wise")
+                assert "mymetric" not in engine.run(data).metrics
 
-            assert "mymetric" not in engine.run(data).metrics
-
-            assert not metric.is_attached(engine, usage="batch_wise")
+                assert not metric.is_attached(engine, usage="batch_wise")
         """
         usage = self._check_usage(usage)
         if engine.has_event_handler(self.completed, usage.COMPLETED):
@@ -423,8 +444,8 @@ class Metric(metaclass=ABCMeta):
         value is written to `engine.state.metrics` dictionary.
 
         Args:
-            engine (Engine): the engine checked from which the metric should be attached
-            usage (str or MetricUsage, optional): the usage of the metric. Valid string values should be
+            engine: the engine checked from which the metric should be attached
+            usage: the usage of the metric. Valid string values should be
                 'epoch_wise' (default) or 'batch_wise'.
         """
         usage = self._check_usage(usage)
@@ -475,16 +496,6 @@ class Metric(metaclass=ABCMeta):
 
         return MetricsLambda(lambda x, y: x % y, self, other)
 
-    def __div__(self, other: Any) -> "MetricsLambda":
-        from ignite.metrics.metrics_lambda import MetricsLambda
-
-        return MetricsLambda(lambda x, y: x.__div__(y), self, other)
-
-    def __rdiv__(self, other: Any) -> "MetricsLambda":
-        from ignite.metrics.metrics_lambda import MetricsLambda
-
-        return MetricsLambda(lambda x, y: x.__div__(y), other, self)
-
     def __truediv__(self, other: Any) -> "MetricsLambda":
         from ignite.metrics.metrics_lambda import MetricsLambda
 
@@ -525,13 +536,15 @@ class Metric(metaclass=ABCMeta):
 
 def sync_all_reduce(*attrs: Any) -> Callable:
     """Helper decorator for distributed configuration to collect instance attribute value
-    across all participating processes.
+    across all participating processes and apply the specified reduction operation.
 
     See :doc:`metrics` on how to use it.
 
     Args:
-        *attrs: attribute names of decorated class
+        attrs: attribute names of decorated class
 
+    .. versionchanged:: 0.4.5
+        - Ability to handle different reduction operations (SUM, MAX, MIN, PRODUCT).
     """
 
     def wrapper(func: Callable) -> Callable:
@@ -542,18 +555,37 @@ def sync_all_reduce(*attrs: Any) -> Callable:
                     "Decorator sync_all_reduce should be used on ignite.metric.Metric class methods only"
                 )
             ws = idist.get_world_size()
-            if len(attrs) > 0 and not self._is_reduced:
-                if ws > 1:
-                    for attr in attrs:
-                        t = getattr(self, attr, None)
-                        if t is not None:
-                            t = idist.all_reduce(t)
-                            self._is_reduced = True
-                            setattr(self, attr, t)
-                else:
-                    self._is_reduced = True
+            unreduced_attrs = {}
+            if len(attrs) > 0 and ws > 1:
+                for attr in attrs:
+                    op_kwargs = {}
+                    if ":" in attr:
+                        attr, op = attr.split(":")
+                        valid_ops = ["MIN", "MAX", "SUM", "PRODUCT"]
+                        if op not in valid_ops:
+                            raise ValueError(f"Reduction operation is not valid (expected : {valid_ops}, got: {op}")
+                        op_kwargs["op"] = op
+                    if attr not in self.__dict__:
+                        raise ValueError(f"Metric {type(self)} has no attribute named `{attr}`.")
+                    t = getattr(self, attr)
+                    if not isinstance(t, (Number, torch.Tensor)):
+                        raise TypeError(
+                            "Attribute provided to sync_all_reduce should be a "
+                            f"number or tensor but `{attr}` has type {type(t)}"
+                        )
+                    unreduced_attrs[attr] = t
+                    # Here `clone` is necessary since `idist.all_reduce` modifies `t` inplace in the case
+                    # `t` is a tensor and its `device` is same as that of the process.
+                    # TODO: Remove this dual behavior of `all_reduce` to always either return a new tensor or
+                    #       modify it in-place.
+                    t_reduced = idist.all_reduce(cast(float, t) if isinstance(t, Number) else t.clone(), **op_kwargs)
+                    setattr(self, attr, t_reduced)
 
-            return func(self, *args, **kwargs)
+            result = func(self, *args, **kwargs)
+
+            for attr, value in unreduced_attrs.items():
+                setattr(self, attr, value)
+            return result
 
         return another_wrapper
 
@@ -566,12 +598,25 @@ def reinit__is_reduced(func: Callable) -> Callable:
 
     See :doc:`metrics` on how to use it.
 
+    Args:
+        func: A callable to reinit.
     """
 
     @wraps(func)
     def wrapper(self: Metric, *args: Any, **kwargs: Any) -> None:
         func(self, *args, **kwargs)
-        self._is_reduced = False
+        if "_result" in self.__dict__:
+            self._result = None  # type: ignore[attr-defined]
 
     setattr(wrapper, "_decorated", True)
     return wrapper
+
+
+def _is_list_of_tensors_or_numbers(x: Sequence[Union[torch.Tensor, float]]) -> bool:
+    return isinstance(x, Sequence) and all([isinstance(t, (torch.Tensor, Number)) for t in x])
+
+
+def _to_batched_tensor(x: Union[torch.Tensor, float], device: Optional[torch.device] = None) -> torch.Tensor:
+    if isinstance(x, torch.Tensor):
+        return x.unsqueeze(dim=0)
+    return torch.tensor([x], device=device)
