@@ -1,7 +1,7 @@
 import math
 import os
 from collections import defaultdict
-from unittest.mock import ANY, MagicMock, Mock, call
+from unittest.mock import ANY, call, MagicMock, patch
 
 import clearml
 import pytest
@@ -13,24 +13,34 @@ import ignite.distributed as idist
 from ignite.contrib.handlers.clearml_logger import (
     ClearMLLogger,
     ClearMLSaver,
+    global_step_from_engine,
     GradsHistHandler,
     GradsScalarHandler,
     OptimizerParamsHandler,
     OutputHandler,
     WeightsHistHandler,
     WeightsScalarHandler,
-    global_step_from_engine,
 )
-from ignite.contrib.handlers.trains_logger import TrainsLogger, TrainsSaver
+
 from ignite.engine import Engine, Events, State
 from ignite.handlers import Checkpoint
 
 
-def test_trains_logger_alias(dirname):
-    with pytest.warns(UserWarning, match="ClearMLSaver: running in bypass mode"):
-        TrainsLogger.set_bypass_mode(True)
-        logger = TrainsLogger(output_uri=dirname)
-        assert isinstance(logger, ClearMLLogger)
+def test_no_clearml():
+    with patch.dict("sys.modules", {"clearml": None, "trains": None}):
+        with pytest.raises(ModuleNotFoundError, match=r"This contrib module requires clearml to be installed."):
+            ClearMLSaver()
+
+        with pytest.raises(ModuleNotFoundError, match=r"This contrib module requires clearml to be installed."):
+            ClearMLLogger()
+
+    with patch.dict("sys.modules", {"clearml.binding.frameworks.tensorflow_bind": None}):
+        with pytest.raises(ModuleNotFoundError, match=r"This contrib module requires clearml to be installed."):
+            ClearMLLogger()
+
+    with patch.dict("sys.modules", {"clearml.binding.frameworks": None, "trains.binding.frameworks": None}):
+        with pytest.raises(ModuleNotFoundError, match=r"This contrib module requires clearml to be installed."):
+            ClearMLSaver.__call__(None, {}, "")
 
 
 def test_optimizer_params_handler_wrong_setup():
@@ -49,7 +59,7 @@ def test_optimizer_params_handler_wrong_setup():
 
 def test_optimizer_params():
 
-    optimizer = torch.optim.SGD([torch.Tensor(0)], lr=0.01)
+    optimizer = torch.optim.SGD([torch.tensor(0.0)], lr=0.01)
     wrapper = OptimizerParamsHandler(optimizer=optimizer, param_name="lr")
     mock_logger = MagicMock(spec=ClearMLLogger)
     mock_logger.clearml_logger = MagicMock()
@@ -137,7 +147,7 @@ def test_output_handler_metric_names(dirname):
     mock_logger = MagicMock(spec=ClearMLLogger)
     mock_logger.clearml_logger = MagicMock()
 
-    with pytest.warns(UserWarning, match=r"ClearMLLogger output_handler can not log metrics value type"):
+    with pytest.warns(UserWarning, match=r"Logger output_handler can not log metrics value type"):
         wrapper(mock_engine, mock_logger, Events.ITERATION_STARTED)
 
     assert mock_logger.clearml_logger.report_scalar.call_count == 1
@@ -183,6 +193,27 @@ def test_output_handler_metric_names(dirname):
         any_order=True,
     )
 
+    # log a torch tensor (ndimension = 0)
+    wrapper = OutputHandler("tag", metric_names="all")
+    mock_logger = MagicMock(spec=ClearMLLogger)
+    mock_logger.clearml_logger = MagicMock()
+
+    mock_engine = MagicMock()
+    mock_engine.state = State(metrics={"a": torch.tensor(12.23), "b": torch.tensor(23.45), "c": torch.tensor(5.01)})
+    mock_engine.state.iteration = 5
+
+    wrapper(mock_engine, mock_logger, Events.ITERATION_STARTED)
+
+    assert mock_logger.clearml_logger.report_scalar.call_count == 3
+    mock_logger.clearml_logger.report_scalar.assert_has_calls(
+        [
+            call(title="tag", series="a", iteration=5, value=torch.tensor(12.23).item()),
+            call(title="tag", series="b", iteration=5, value=torch.tensor(23.45).item()),
+            call(title="tag", series="c", iteration=5, value=torch.tensor(5.01).item()),
+        ],
+        any_order=True,
+    )
+
 
 def test_output_handler_both(dirname):
 
@@ -262,6 +293,32 @@ def test_output_handler_with_global_step_from_engine():
     )
 
 
+def test_output_handler_state_attrs():
+    wrapper = OutputHandler("tag", state_attributes=["alpha", "beta", "gamma"])
+    mock_logger = MagicMock(spec=ClearMLLogger)
+    mock_logger.clearml_logger = MagicMock()
+
+    mock_engine = MagicMock()
+    mock_engine.state = State()
+    mock_engine.state.iteration = 5
+    mock_engine.state.alpha = 3.899
+    mock_engine.state.beta = torch.tensor(12.0)
+    mock_engine.state.gamma = torch.tensor([21.0, 6.0])
+
+    wrapper(mock_engine, mock_logger, Events.ITERATION_STARTED)
+
+    assert mock_logger.clearml_logger.report_scalar.call_count == 4
+    mock_logger.clearml_logger.report_scalar.assert_has_calls(
+        [
+            call(title="tag", series="alpha", iteration=5, value=3.899),
+            call(title="tag", series="beta", iteration=5, value=12.0),
+            call(title="tag/gamma", series="0", iteration=5, value=21.0),
+            call(title="tag/gamma", series="1", iteration=5, value=6.0),
+        ],
+        any_order=True,
+    )
+
+
 def test_output_handler_with_global_step_transform():
     def global_step_transform(*args, **kwargs):
         return 10
@@ -284,16 +341,7 @@ def test_output_handler_with_global_step_transform():
 
 def test_weights_scalar_handler_wrong_setup():
 
-    with pytest.raises(TypeError, match="Argument model should be of type torch.nn.Module"):
-        WeightsScalarHandler(None)
-
     model = MagicMock(spec=torch.nn.Module)
-    with pytest.raises(TypeError, match="Argument reduction should be callable"):
-        WeightsScalarHandler(model, reduction=123)
-
-    with pytest.raises(TypeError, match="Output of the reduction function should be a scalar"):
-        WeightsScalarHandler(model, reduction=lambda x: x)
-
     wrapper = WeightsScalarHandler(model)
     mock_logger = MagicMock()
     mock_engine = MagicMock()
@@ -334,11 +382,10 @@ def test_weights_scalar_handler(dummy_model_factory):
     _test(tag="tag")
 
 
-def test_weights_scalar_handler_frozen_layers(dummy_model_factory):
+def test_weights_scalar_handler_whitelist(dummy_model_factory):
+    model = dummy_model_factory()
 
-    model = dummy_model_factory(with_grads=True, with_frozen_layer=True)
-
-    wrapper = WeightsScalarHandler(model)
+    wrapper = WeightsScalarHandler(model, whitelist=["fc2.weight"])
     mock_logger = MagicMock(spec=ClearMLLogger)
     mock_logger.clearml_logger = MagicMock()
 
@@ -346,33 +393,43 @@ def test_weights_scalar_handler_frozen_layers(dummy_model_factory):
     mock_engine.state = State()
     mock_engine.state.epoch = 5
 
+    wrapper(mock_engine, mock_logger, Events.EPOCH_STARTED)
+    mock_logger.clearml_logger.report_scalar.assert_called_once_with(
+        title="weights_norm/fc2", value=ANY, series="weight", iteration=mock_engine.state.epoch
+    )
+    mock_logger.clearml_logger.report_scalar.reset_mock()
+
+    wrapper = WeightsScalarHandler(model, tag="model", whitelist=["fc1"])
     wrapper(mock_engine, mock_logger, Events.EPOCH_STARTED)
 
     mock_logger.clearml_logger.report_scalar.assert_has_calls(
         [
-            call(title="weights_norm/fc2", series="weight", iteration=5, value=12.0),
-            call(title="weights_norm/fc2", series="bias", iteration=5, value=math.sqrt(12.0)),
+            call(title="model/weights_norm/fc1", value=ANY, series="weight", iteration=mock_engine.state.epoch),
+            call(title="model/weights_norm/fc1", value=ANY, series="bias", iteration=mock_engine.state.epoch),
         ],
         any_order=True,
     )
+    assert mock_logger.clearml_logger.report_scalar.call_count == 2
+    mock_logger.clearml_logger.report_scalar.reset_mock()
 
-    with pytest.raises(AssertionError):
-        mock_logger.clearml_logger.report_scalar.assert_has_calls(
-            [
-                call(title="weights_norm/fc1", series="weight", iteration=5, value=12.0),
-                call(title="weights_norm/fc1", series="bias", iteration=5, value=math.sqrt(12.0)),
-            ],
-            any_order=True,
-        )
+    def weight_selector(n, _):
+        return "bias" in n
 
+    wrapper = WeightsScalarHandler(model, tag="model", whitelist=weight_selector)
+    wrapper(mock_engine, mock_logger, Events.EPOCH_STARTED)
+
+    mock_logger.clearml_logger.report_scalar.assert_has_calls(
+        [
+            call(title="model/weights_norm/fc1", value=ANY, series="bias", iteration=mock_engine.state.epoch),
+            call(title="model/weights_norm/fc2", value=ANY, series="bias", iteration=mock_engine.state.epoch),
+        ],
+        any_order=True,
+    )
     assert mock_logger.clearml_logger.report_scalar.call_count == 2
 
 
 def test_weights_hist_handler_wrong_setup():
 
-    with pytest.raises(TypeError, match="Argument model should be of type torch.nn.Module"):
-        WeightsHistHandler(None)
-
     model = MagicMock(spec=torch.nn.Module)
     wrapper = WeightsHistHandler(model)
     mock_logger = MagicMock()
@@ -414,11 +471,10 @@ def test_weights_hist_handler(dummy_model_factory):
     _test(tag="tag")
 
 
-def test_weights_hist_handler_frozen_layers(dummy_model_factory):
+def test_weights_hist_handler_whitelist(dummy_model_factory):
+    model = dummy_model_factory()
 
-    model = dummy_model_factory(with_grads=True, with_frozen_layer=True)
-
-    wrapper = WeightsHistHandler(model)
+    wrapper = WeightsHistHandler(model, whitelist=["fc2.weight"])
     mock_logger = MagicMock(spec=ClearMLLogger)
     mock_logger.grad_helper = MagicMock()
 
@@ -426,36 +482,44 @@ def test_weights_hist_handler_frozen_layers(dummy_model_factory):
     mock_engine.state = State()
     mock_engine.state.epoch = 5
 
+    wrapper(mock_engine, mock_logger, Events.EPOCH_STARTED)
+    mock_logger.grad_helper.add_histogram.assert_called_once_with(
+        title="weights_fc2", hist_data=ANY, series="weight", step=5
+    )
+    mock_logger.grad_helper.add_histogram.reset_mock()
+
+    wrapper = WeightsHistHandler(model, tag="model", whitelist=["fc1"])
     wrapper(mock_engine, mock_logger, Events.EPOCH_STARTED)
 
     mock_logger.grad_helper.add_histogram.assert_has_calls(
         [
-            call(title="weights_fc2", hist_data=ANY, series="weight", step=5),
-            call(title="weights_fc2", hist_data=ANY, series="bias", step=5),
+            call(title="model/weights_fc1", hist_data=ANY, series="weight", step=5),
+            call(title="model/weights_fc1", hist_data=ANY, series="bias", step=5),
         ],
         any_order=True,
     )
+    assert mock_logger.grad_helper.add_histogram.call_count == 2
+    mock_logger.grad_helper.add_histogram.reset_mock()
 
-    with pytest.raises(AssertionError):
-        mock_logger.grad_helper.add_histogram.assert_has_calls(
-            [
-                call(title="weights_fc1", hist_data=ANY, series="weight", step=5),
-                call(title="weights_fc1", hist_data=ANY, series="bias", step=5),
-            ],
-            any_order=True,
-        )
+    def weight_selector(n, _):
+        return "bias" in n
+
+    wrapper = WeightsHistHandler(model, tag="model", whitelist=weight_selector)
+    wrapper(mock_engine, mock_logger, Events.EPOCH_STARTED)
+
+    mock_logger.grad_helper.add_histogram.assert_has_calls(
+        [
+            call(title="model/weights_fc1", hist_data=ANY, series="bias", step=5),
+            call(title="model/weights_fc2", hist_data=ANY, series="bias", step=5),
+        ],
+        any_order=True,
+    )
     assert mock_logger.grad_helper.add_histogram.call_count == 2
 
 
 def test_grads_scalar_handler_wrong_setup():
 
-    with pytest.raises(TypeError, match="Argument model should be of type torch.nn.Module"):
-        GradsScalarHandler(None)
-
     model = MagicMock(spec=torch.nn.Module)
-    with pytest.raises(TypeError, match="Argument reduction should be callable"):
-        GradsScalarHandler(model, reduction=123)
-
     wrapper = GradsScalarHandler(model)
     mock_logger = MagicMock()
     mock_engine = MagicMock()
@@ -501,41 +565,54 @@ def test_grads_scalar_handler(dummy_model_factory, norm_mock):
     _test(tag="tag")
 
 
-def test_grads_scalar_handler_frozen_layers(dummy_model_factory, norm_mock):
-    model = dummy_model_factory(with_grads=True, with_frozen_layer=True)
+def test_grads_scalar_handler_whitelist(dummy_model_factory):
+    model = dummy_model_factory()
 
-    wrapper = GradsScalarHandler(model, reduction=norm_mock)
+    wrapper = GradsScalarHandler(model, whitelist=["fc2.weight"])
     mock_logger = MagicMock(spec=ClearMLLogger)
     mock_logger.clearml_logger = MagicMock()
 
     mock_engine = MagicMock()
     mock_engine.state = State()
     mock_engine.state.epoch = 5
-    norm_mock.reset_mock()
 
+    wrapper(mock_engine, mock_logger, Events.EPOCH_STARTED)
+    mock_logger.clearml_logger.report_scalar.assert_called_once_with(
+        title="grads_norm/fc2", value=ANY, series="weight", iteration=mock_engine.state.epoch
+    )
+    mock_logger.clearml_logger.report_scalar.reset_mock()
+
+    wrapper = GradsScalarHandler(model, tag="model", whitelist=["fc1"])
     wrapper(mock_engine, mock_logger, Events.EPOCH_STARTED)
 
     mock_logger.clearml_logger.report_scalar.assert_has_calls(
         [
-            call(title="grads_norm/fc2", value=ANY, series="weight", iteration=mock_engine.state.epoch),
-            call(title="grads_norm/fc2", value=ANY, series="bias", iteration=mock_engine.state.epoch),
+            call(title="model/grads_norm/fc1", value=ANY, series="weight", iteration=mock_engine.state.epoch),
+            call(title="model/grads_norm/fc1", value=ANY, series="bias", iteration=mock_engine.state.epoch),
+        ],
+        any_order=True,
+    )
+    assert mock_logger.clearml_logger.report_scalar.call_count == 2
+    mock_logger.clearml_logger.report_scalar.reset_mock()
+
+    def weight_selector(n, _):
+        return "bias" in n
+
+    wrapper = GradsScalarHandler(model, tag="model", whitelist=weight_selector)
+    wrapper(mock_engine, mock_logger, Events.EPOCH_STARTED)
+
+    mock_logger.clearml_logger.report_scalar.assert_has_calls(
+        [
+            call(title="model/grads_norm/fc1", value=ANY, series="bias", iteration=mock_engine.state.epoch),
+            call(title="model/grads_norm/fc2", value=ANY, series="bias", iteration=mock_engine.state.epoch),
         ],
         any_order=True,
     )
-
-    with pytest.raises(AssertionError):
-        mock_logger.clearml_logger.report_scalar.assert_has_calls(
-            [call(title="grads_norm/fc1", value=ANY, iteration=5), call("grads_norm/fc1", ANY, 5)], any_order=True
-        )
     assert mock_logger.clearml_logger.report_scalar.call_count == 2
-    assert norm_mock.call_count == 2
 
 
 def test_grads_hist_handler_wrong_setup():
 
-    with pytest.raises(TypeError, match="Argument model should be of type torch.nn.Module"):
-        GradsHistHandler(None)
-
     model = MagicMock(spec=torch.nn.Module)
     wrapper = GradsHistHandler(model)
     mock_logger = MagicMock()
@@ -576,10 +653,10 @@ def test_grads_hist_handler(dummy_model_factory):
     _test(tag="tag")
 
 
-def test_grads_hist_frozen_layers(dummy_model_factory):
-    model = dummy_model_factory(with_grads=True, with_frozen_layer=True)
+def test_grads_hist_handler_whitelist(dummy_model_factory):
+    model = dummy_model_factory()
 
-    wrapper = GradsHistHandler(model)
+    wrapper = GradsHistHandler(model, whitelist=["fc2.weight"])
     mock_logger = MagicMock(spec=ClearMLLogger)
     mock_logger.grad_helper = MagicMock()
 
@@ -587,25 +664,39 @@ def test_grads_hist_frozen_layers(dummy_model_factory):
     mock_engine.state = State()
     mock_engine.state.epoch = 5
 
+    wrapper(mock_engine, mock_logger, Events.EPOCH_STARTED)
+    mock_logger.grad_helper.add_histogram.assert_called_once_with(
+        title="grads_fc2", hist_data=ANY, series="weight", step=5
+    )
+    mock_logger.grad_helper.reset_mock()
+
+    wrapper = GradsHistHandler(model, tag="model", whitelist=["fc1"])
     wrapper(mock_engine, mock_logger, Events.EPOCH_STARTED)
 
-    assert mock_logger.grad_helper.add_histogram.call_count == 2
     mock_logger.grad_helper.add_histogram.assert_has_calls(
         [
-            call(title="grads_fc2", hist_data=ANY, series="weight", step=5),
-            call(title="grads_fc2", hist_data=ANY, series="bias", step=5),
+            call(title="model/grads_fc1", hist_data=ANY, series="weight", step=5),
+            call(title="model/grads_fc1", hist_data=ANY, series="bias", step=5),
         ],
         any_order=True,
     )
+    assert mock_logger.grad_helper.add_histogram.call_count == 2
+    mock_logger.grad_helper.reset_mock()
 
-    with pytest.raises(AssertionError):
-        mock_logger.grad_helper.add_histogram.assert_has_calls(
-            [
-                call(title="grads_fc1", hist_data=ANY, series="weight", step=5),
-                call(title="grads_fc1", hist_data=ANY, series="bias", step=5),
-            ],
-            any_order=True,
-        )
+    def weight_selector(n, _):
+        return "bias" in n
+
+    wrapper = GradsHistHandler(model, tag="model", whitelist=weight_selector)
+    wrapper(mock_engine, mock_logger, Events.EPOCH_STARTED)
+
+    mock_logger.grad_helper.add_histogram.assert_has_calls(
+        [
+            call(title="model/grads_fc1", hist_data=ANY, series="bias", step=5),
+            call(title="model/grads_fc2", hist_data=ANY, series="bias", step=5),
+        ],
+        any_order=True,
+    )
+    assert mock_logger.grad_helper.add_histogram.call_count == 2
 
 
 def test_integration(dirname):
@@ -627,7 +718,8 @@ def test_integration(dirname):
 
         def dummy_handler(engine, logger, event_name):
             global_step = engine.state.get_event_attrib_value(event_name)
-            logger.clearml_logger.report_scalar(title="", series="", value="test_value", iteration=global_step)
+            test_value = 0.3  # example
+            logger.clearml_logger.report_scalar(title="", series="", value=test_value, iteration=global_step)
 
         logger.attach(trainer, log_handler=dummy_handler, event_name=Events.EPOCH_COMPLETED)
 
@@ -654,19 +746,54 @@ def test_integration_as_context_manager(dirname):
 
             def dummy_handler(engine, logger, event_name):
                 global_step = engine.state.get_event_attrib_value(event_name)
-                logger.clearml_logger.report_scalar(title="", series="", value="test_value", iteration=global_step)
+                test_value = 0.3  # example
+                logger.clearml_logger.report_scalar(title="", series="", value=test_value, iteration=global_step)
 
             clearml_logger.attach(trainer, log_handler=dummy_handler, event_name=Events.EPOCH_COMPLETED)
 
             trainer.run(data, max_epochs=n_epochs)
 
 
+def test_clearml_logger_getattr_method(dirname):
+
+    with pytest.warns(UserWarning, match="ClearMLSaver: running in bypass mode"):
+        ClearMLLogger.set_bypass_mode(True)
+
+        logger = ClearMLLogger(output_uri=dirname)
+
+        # Create a mock clearml.Logger() object
+        mock_logger = MagicMock()
+        logger.clearml_logger = mock_logger
+
+        # Test a method called by __getattr__ calls the corresponding method of the mock project.
+        logger.report_single_value("accuracy", 0.72)
+        mock_logger.report_single_value.assert_called_once_with("accuracy", 0.72)
+
+        # Test a method called by __getattr__ calls the corresponding classmethod of the mock project's class.
+        logger.current_logger()
+        mock_logger.current_logger.assert_called_once()
+
+        logger.close()
+
+
+def test_clearml_logger_get_task_bypass(dirname):
+
+    with pytest.warns(UserWarning, match="ClearMLSaver: running in bypass mode"):
+        ClearMLLogger.set_bypass_mode(True)
+
+        with ClearMLLogger(output_uri=dirname) as clearml_logger:
+            task = clearml_logger.get_task()
+            assert isinstance(task, clearml.Task)
+            assert task == clearml.Task.current_task()
+            task.close()
+
+
 def test_clearml_disk_saver_integration():
     model = torch.nn.Module()
     to_save_serializable = {"model": model}
     with pytest.warns(UserWarning, match="ClearMLSaver created a temporary checkpoints directory"):
         mock_logger = MagicMock(spec=ClearMLLogger)
-        clearml.Task.current_task = Mock(return_value=object())
+        clearml.Task.current_task = MagicMock(spec=clearml.Task)
         clearml_saver = ClearMLSaver(mock_logger)
         clearml.binding.frameworks.WeightsFileHandler.create_output_model = MagicMock()
 
@@ -690,7 +817,7 @@ def test_clearml_disk_saver_integration_no_logger():
     to_save_serializable = {"model": model}
 
     with pytest.warns(UserWarning, match="ClearMLSaver created a temporary checkpoints directory"):
-        clearml.Task.current_task = Mock(return_value=object())
+        clearml.Task.current_task = MagicMock(spec=clearml.Task)
         clearml.binding.frameworks.WeightsFileHandler.create_output_model = MagicMock()
         clearml_saver = ClearMLSaver()
         checkpoint = Checkpoint(to_save=to_save_serializable, save_handler=clearml_saver, n_saved=1)
@@ -802,7 +929,7 @@ class DummyModel(torch.nn.Module):
 def _test_save_model_optimizer_lr_scheduler_with_state_dict(device, on_zero_rank=False):
 
     if idist.get_rank() == 0:
-        clearml.Task.current_task = Mock(return_value=object())
+        clearml.Task.current_task = MagicMock(spec=clearml.Task)
         clearml.binding.frameworks.WeightsFileHandler.create_output_model = MagicMock()
 
     torch.manual_seed(23)
@@ -846,7 +973,7 @@ def _test_save_model_optimizer_lr_scheduler_with_state_dict(device, on_zero_rank
 
     saved_objects = sorted(os.listdir(clearml_saver.dirname))
     # saved object is ['PREFIX_checkpoint_3.pt', ]
-    saved_checkpoint = os.path.join(clearml_saver.dirname, saved_objects[0])
+    saved_checkpoint = clearml_saver.dirname / saved_objects[0]
 
     if idist.has_xla_support:
         device = "cpu"
@@ -888,18 +1015,21 @@ def _test_save_model_optimizer_lr_scheduler_with_state_dict(device, on_zero_rank
 
 @pytest.mark.distributed
 @pytest.mark.skipif(not idist.has_native_dist_support, reason="Skip if no native dist support")
-def test_distrib_cpu(distributed_context_single_node_gloo):
-    _test_save_model_optimizer_lr_scheduler_with_state_dict("cpu")
-    _test_save_model_optimizer_lr_scheduler_with_state_dict("cpu", on_zero_rank=True)
+def test_distrib_gloo_cpu_or_gpu(distributed_context_single_node_gloo):
+
+    device = idist.device()
+    _test_save_model_optimizer_lr_scheduler_with_state_dict(device)
+    _test_save_model_optimizer_lr_scheduler_with_state_dict(device, on_zero_rank=True)
 
 
 @pytest.mark.distributed
 @pytest.mark.skipif(not idist.has_native_dist_support, reason="Skip if no native dist support")
 @pytest.mark.skipif(torch.cuda.device_count() < 1, reason="Skip if no GPU")
-def test_distrib_gpu(distributed_context_single_node_nccl):
+def test_distrib_nccl_gpu(distributed_context_single_node_nccl):
+
     device = idist.device()
     _test_save_model_optimizer_lr_scheduler_with_state_dict(device)
-    _test_save_model_optimizer_lr_scheduler_with_state_dict("cpu", on_zero_rank=True)
+    _test_save_model_optimizer_lr_scheduler_with_state_dict(device, on_zero_rank=True)
 
 
 @pytest.mark.tpu
