@@ -17,6 +17,11 @@ def test_deprecated_callable_events_class():
         class CustomEvents(CallableEvents, Enum):
             TEST_EVENT = "test_event"
 
+            def __new__(cls, value: str) -> "CallableEvents":
+                obj = CallableEvents.__new__(cls)
+                obj._value_ = value
+                return obj
+
         with pytest.raises(TypeError, match=r"Value at \d of event_names should be a str or EventEnum"):
             engine.register_events(*CustomEvents)
 
@@ -141,12 +146,62 @@ def test_custom_events_with_events_list():
 
 
 def test_callable_events_with_wrong_inputs():
+    def ef(e, i):
+        return 1
 
-    with pytest.raises(ValueError, match=r"Only one of the input arguments should be specified"):
-        Events.ITERATION_STARTED()
-
-    with pytest.raises(ValueError, match=r"Only one of the input arguments should be specified"):
-        Events.ITERATION_STARTED(event_filter="123", every=12)
+    expected_raise = {
+        # event_filter, every, once, before, after
+        (None, None, None, None, None): True,  # raises ValueError
+        (ef, None, None, None, None): False,
+        (None, 2, None, None, None): False,
+        (ef, 2, None, None, None): True,
+        (None, None, 2, None, None): False,
+        (ef, None, 2, None, None): True,
+        (None, 2, 2, None, None): True,
+        (ef, 2, 2, None, None): True,
+        (None, None, None, 30, None): False,
+        (ef, None, None, 30, None): True,
+        (None, 2, None, 30, None): False,
+        (ef, 2, None, 30, None): True,
+        (None, None, 2, 30, None): True,
+        (ef, None, 2, 30, None): True,
+        (None, 2, 2, 30, None): True,
+        (ef, 2, 2, 30, None): True,
+        # event_filter, every, once, before, after
+        (None, None, None, None, 10): False,
+        (ef, None, None, None, 10): True,
+        (None, 2, None, None, 10): False,
+        (ef, 2, None, None, 10): True,
+        (None, None, 2, None, 10): True,
+        (ef, None, 2, None, 10): True,
+        (None, 2, 2, None, 10): True,
+        (ef, 2, 2, None, 10): True,
+        (None, None, None, 25, 8): False,
+        (ef, None, None, 25, 8): True,
+        (None, 2, None, 25, 8): False,
+        (ef, 2, None, 25, 8): True,
+        (None, None, 2, 25, 8): True,
+        (ef, None, 2, 25, 8): True,
+        (None, 2, 2, 25, 8): True,
+        (ef, 2, 2, 25, 8): True,
+    }
+    for event_filter in [None, ef]:
+        for every in [None, 2]:
+            for once in [None, 2]:
+                for before, after in [(None, None), (None, 10), (30, None), (25, 8)]:
+                    if expected_raise[(event_filter, every, once, before, after)]:
+                        with pytest.raises(
+                            ValueError,
+                            match=r"Only one of the input arguments should be specified, "
+                            "except before, after and every",
+                        ):
+                            Events.ITERATION_STARTED(
+                                event_filter=event_filter, once=once, every=every, before=before, after=after
+                            )
+                    else:
+                        Events.ITERATION_STARTED(
+                            event_filter=event_filter, once=once, every=every, before=before, after=after
+                        )
 
     with pytest.raises(TypeError, match=r"Argument event_filter should be a callable"):
         Events.ITERATION_STARTED(event_filter="123")
@@ -154,49 +209,82 @@ def test_callable_events_with_wrong_inputs():
     with pytest.raises(ValueError, match=r"Argument every should be integer and greater than zero"):
         Events.ITERATION_STARTED(every=-1)
 
+    with pytest.raises(
+        ValueError, match=r"Argument once should either be a positive integer or a list of positive integers, got .+"
+    ):
+        Events.ITERATION_STARTED(once=-1)
+
+    with pytest.raises(
+        ValueError, match=r"Argument once should either be a positive integer or a list of positive integers, got .+"
+    ):
+        Events.ITERATION_STARTED(once=[1, 10.0, "pytorch"])
+
+    with pytest.raises(
+        ValueError, match=r"Argument once should either be a positive integer or a list of positive integers, got .+"
+    ):
+        Events.ITERATION_STARTED(once=[])
+
+    with pytest.raises(ValueError, match=r"Argument before should be integer and greater or equal to zero"):
+        Events.ITERATION_STARTED(before=-1)
+
+    with pytest.raises(ValueError, match=r"Argument after should be integer and greater or equal to zero"):
+        Events.ITERATION_STARTED(after=-1)
+
     with pytest.raises(ValueError, match=r"but will be called with"):
         Events.ITERATION_STARTED(event_filter=lambda x: x)
 
+    with pytest.warns(UserWarning, match=r"default_event_filter is deprecated and will be removed"):
+        Events.default_event_filter(None, None)
 
-def test_callable_events():
 
-    assert isinstance(Events.ITERATION_STARTED.value, str)
+@pytest.mark.parametrize(
+    "event",
+    [
+        Events.ITERATION_STARTED,
+        Events.ITERATION_COMPLETED,
+        Events.EPOCH_STARTED,
+        Events.EPOCH_COMPLETED,
+        Events.GET_BATCH_STARTED,
+        Events.GET_BATCH_COMPLETED,
+        Events.STARTED,
+        Events.COMPLETED,
+    ],
+)
+def test_callable_events(event):
 
-    def foo(engine, event):
+    assert isinstance(event.value, str)
+
+    def foo(engine, _):
         return True
 
-    ret = Events.ITERATION_STARTED(event_filter=foo)
+    ret = event(event_filter=foo)
     assert isinstance(ret, CallableEventWithFilter)
-    assert ret == Events.ITERATION_STARTED
+    assert ret == event
     assert ret.filter == foo
-    assert isinstance(Events.ITERATION_STARTED.value, str)
+    assert event.name in f"{ret}"
 
-    # assert ret in Events
-    assert Events.ITERATION_STARTED.name in f"{ret}"
-    # assert ret in State.event_to_attr
-
-    ret = Events.ITERATION_STARTED(every=10)
+    ret = event(every=10)
     assert isinstance(ret, CallableEventWithFilter)
-    assert ret == Events.ITERATION_STARTED
+    assert ret == event
     assert ret.filter is not None
+    assert event.name in f"{ret}"
 
-    # assert ret in Events
-    assert Events.ITERATION_STARTED.name in f"{ret}"
-    # assert ret in State.event_to_attr
-
-    ret = Events.ITERATION_STARTED(once=10)
+    ret = event(once=10)
     assert isinstance(ret, CallableEventWithFilter)
-    assert ret == Events.ITERATION_STARTED
+    assert ret == event
     assert ret.filter is not None
+    assert event.name in f"{ret}"
 
-    # assert ret in Events
-    assert Events.ITERATION_STARTED.name in f"{ret}"
-    # assert ret in State.event_to_attr
+    ret = event(once=[1, 10])
+    assert isinstance(ret, CallableEventWithFilter)
+    assert ret == event
+    assert ret.filter is not None
+    assert event.name in f"{ret}"
 
-    def _attach(e1, e2):
-        assert id(e1) != id(e2)
-
-    _attach(Events.ITERATION_STARTED(every=10), Events.ITERATION_COMPLETED(every=10))
+    ret = event
+    assert isinstance(ret, CallableEventWithFilter)
+    assert ret.filter is None
+    assert event.name in f"{ret}"
 
 
 def test_callable_events_every_eq_one():
@@ -221,8 +309,7 @@ def test_has_handler_on_callable_events():
     engine.add_event_handler(Events.EPOCH_COMPLETED(every=3), bar)
     assert engine.has_event_handler(bar)
     assert engine.has_event_handler(bar, Events.EPOCH_COMPLETED)
-
-    engine.has_event_handler(bar, Events.EPOCH_COMPLETED(every=3))
+    assert engine.has_event_handler(bar, Events.EPOCH_COMPLETED(every=3))
 
 
 def test_remove_event_handler_on_callable_events():
@@ -261,15 +348,9 @@ def _test_every_event_filter_with_engine(device="cpu"):
 
         engine = Engine(lambda e, b: b)
 
-        counter = [
-            0,
-        ]
-        counter_every = [
-            0,
-        ]
-        num_calls = [
-            0,
-        ]
+        counter = [0]
+        counter_every = [0]
+        num_calls = [0]
 
         @engine.on(event_name(every=every))
         def assert_every(engine):
@@ -306,38 +387,141 @@ def test_every_event_filter_with_engine():
     _test_every_event_filter_with_engine()
 
 
-def test_once_event_filter_with_engine():
-    def _test(event_name, event_attr):
+@pytest.mark.parametrize(
+    "event_name, event_attr, before, expect_calls",
+    [
+        (Events.ITERATION_COMPLETED, "iteration", 0, 0),
+        (Events.ITERATION_COMPLETED, "iteration", 300, 299),
+        (Events.ITERATION_COMPLETED, "iteration", 501, 500),
+        (Events.EPOCH_COMPLETED, "epoch", 0, 0),
+        (Events.EPOCH_COMPLETED, "epoch", 3, 2),
+        (Events.EPOCH_COMPLETED, "epoch", 6, 5),
+    ],
+)
+def test_before_event_filter_with_engine(event_name, event_attr, before, expect_calls):
 
-        engine = Engine(lambda e, b: b)
+    data = range(100)
 
-        once = 2
-        counter = [
-            0,
-        ]
-        num_calls = [
-            0,
-        ]
+    engine = Engine(lambda e, b: 1)
+    num_calls = 0
 
-        @engine.on(event_name(once=once))
-        def assert_once(engine):
-            assert getattr(engine.state, event_attr) == once
-            num_calls[0] += 1
+    @engine.on(event_name(before=before))
+    def _before_event():
+        nonlocal num_calls
+        num_calls += 1
+        assert getattr(engine.state, event_attr) < before
 
-        @engine.on(event_name)
-        def assert_(engine):
-            counter[0] += 1
-            assert getattr(engine.state, event_attr) == counter[0]
+    engine.run(data, max_epochs=5)
+    assert num_calls == expect_calls
 
-        d = list(range(100))
-        engine.run(d, max_epochs=5)
 
-        assert num_calls[0] == 1
+@pytest.mark.parametrize(
+    "event_name, event_attr, after, expect_calls",
+    [
+        (Events.ITERATION_STARTED, "iteration", 0, 500),
+        (Events.ITERATION_COMPLETED, "iteration", 300, 200),
+        (Events.ITERATION_COMPLETED, "iteration", 500, 0),
+        (Events.EPOCH_STARTED, "epoch", 0, 5),
+        (Events.EPOCH_COMPLETED, "epoch", 3, 2),
+        (Events.EPOCH_COMPLETED, "epoch", 5, 0),
+    ],
+)
+def test_after_event_filter_with_engine(event_name, event_attr, after, expect_calls):
 
-    _test(Events.ITERATION_STARTED, "iteration")
-    _test(Events.ITERATION_COMPLETED, "iteration")
-    _test(Events.EPOCH_STARTED, "epoch")
-    _test(Events.EPOCH_COMPLETED, "epoch")
+    data = range(100)
+
+    engine = Engine(lambda e, b: 1)
+    num_calls = 0
+
+    @engine.on(event_name(after=after))
+    def _after_event():
+        nonlocal num_calls
+        num_calls += 1
+        assert getattr(engine.state, event_attr) > after
+
+    engine.run(data, max_epochs=5)
+    assert num_calls == expect_calls
+
+
+@pytest.mark.parametrize(
+    "event_name, event_attr, before, after, expect_calls",
+    [(Events.ITERATION_STARTED, "iteration", 300, 100, 199), (Events.EPOCH_COMPLETED, "epoch", 4, 1, 2)],
+)
+def test_before_and_after_event_filter_with_engine(event_name, event_attr, before, after, expect_calls):
+
+    data = range(100)
+
+    engine = Engine(lambda e, b: 1)
+    num_calls = 0
+
+    @engine.on(event_name(before=before, after=after))
+    def _before_and_after_event():
+        nonlocal num_calls
+        num_calls += 1
+        assert getattr(engine.state, event_attr) > after
+
+    engine.run(data, max_epochs=5)
+    assert num_calls == expect_calls
+
+
+@pytest.mark.parametrize(
+    "event_name, event_attr, every, before, after, expect_calls",
+    [(Events.ITERATION_STARTED, "iteration", 5, 25, 8, 4), (Events.EPOCH_COMPLETED, "epoch", 2, 5, 1, 2)],
+)
+def test_every_before_and_after_event_filter_with_engine(event_name, event_attr, every, before, after, expect_calls):
+
+    data = range(100)
+
+    engine = Engine(lambda e, b: 1)
+    num_calls = 0
+
+    @engine.on(event_name(every=every, before=before, after=after))
+    def _every_before_and_after_event():
+        assert getattr(engine.state, event_attr) > after
+        assert getattr(engine.state, event_attr) < before
+        assert ((getattr(engine.state, event_attr) - after - 1) % every) == 0
+        nonlocal num_calls
+        num_calls += 1
+
+    engine.run(data, max_epochs=5)
+    assert num_calls == expect_calls
+
+
+@pytest.mark.parametrize(
+    "event_name, event_attr, once, expect_calls",
+    [
+        (Events.ITERATION_STARTED, "iteration", 2, 1),
+        (Events.ITERATION_COMPLETED, "iteration", 2, 1),
+        (Events.EPOCH_STARTED, "epoch", 2, 1),
+        (Events.EPOCH_COMPLETED, "epoch", 2, 1),
+        (Events.ITERATION_STARTED, "iteration", [1, 5], 2),
+        (Events.ITERATION_COMPLETED, "iteration", [1, 5], 2),
+        (Events.EPOCH_STARTED, "epoch", [1, 5], 2),
+        (Events.EPOCH_COMPLETED, "epoch", [1, 5], 2),
+    ],
+)
+def test_once_event_filter(event_name, event_attr, once, expect_calls):
+
+    data = list(range(100))
+
+    engine = Engine(lambda e, b: b)
+    num_calls = [0]
+    counter = [0]
+
+    test_once = [once] if isinstance(once, int) else once
+
+    @engine.on(event_name(once=once))
+    def assert_once(engine):
+        assert getattr(engine.state, event_attr) in test_once
+        num_calls[0] += 1
+
+    @engine.on(event_name)
+    def assert_(engine):
+        counter[0] += 1
+        assert getattr(engine.state, event_attr) == counter[0]
+
+    engine.run(data, max_epochs=10)
+    assert num_calls[0] == expect_calls
 
 
 def test_custom_event_filter_with_engine():
@@ -353,9 +537,7 @@ def test_custom_event_filter_with_engine():
 
         engine = Engine(lambda e, b: b)
 
-        num_calls = [
-            0,
-        ]
+        num_calls = [0]
 
         @engine.on(event_name(event_filter=custom_event_filter))
         def assert_on_special_event(engine):
@@ -384,9 +566,7 @@ def test_callable_event_bad_behaviour():
 
     # Check bad behaviour
     engine = Engine(lambda e, b: b)
-    counter = [
-        0,
-    ]
+    counter = [0]
 
     # Modify events
     Events.ITERATION_STARTED(event_filter=custom_event_filter)
@@ -436,9 +616,7 @@ def test_custom_callable_events_with_engine():
         engine = Engine(update_fn)
         engine.register_events(*CustomEvents, event_to_attr=event_to_attr)
 
-        num_calls = [
-            0,
-        ]
+        num_calls = [0]
 
         @engine.on(event_name(event_filter=custom_event_filter))
         def assert_on_special_event(engine):
@@ -464,7 +642,7 @@ def _test_every_event_filter_with_engine_with_dataloader(device):
             data,
             batch_size=batch_size,
             num_workers=num_workers,
-            pin_memory="cuda" in device,
+            pin_memory="cuda" in torch.device(device).type,
             drop_last=True,
             shuffle=True,
         )
@@ -498,16 +676,19 @@ def test_every_event_filter_with_engine_with_dataloader():
 
 @pytest.mark.distributed
 @pytest.mark.skipif(not idist.has_native_dist_support, reason="Skip if no native dist support")
-def test_distrib_cpu(distributed_context_single_node_gloo):
-    _test_every_event_filter_with_engine()
-    _test_every_event_filter_with_engine_with_dataloader("cpu")
+def test_distrib_gloo_cpu_or_gpu(distributed_context_single_node_gloo):
+
+    device = idist.device()
+    _test_every_event_filter_with_engine(device)
+    _test_every_event_filter_with_engine_with_dataloader(device)
 
 
 @pytest.mark.distributed
 @pytest.mark.skipif(not idist.has_native_dist_support, reason="Skip if no native dist support")
 @pytest.mark.skipif(torch.cuda.device_count() < 1, reason="Skip if no GPU")
-def test_distrib_gpu(distributed_context_single_node_nccl):
-    device = f"cuda:{distributed_context_single_node_nccl['local_rank']}"
+def test_distrib_nccl_gpu(distributed_context_single_node_nccl):
+
+    device = idist.device()
     _test_every_event_filter_with_engine(device)
     _test_every_event_filter_with_engine_with_dataloader(device)
 
@@ -549,3 +730,5 @@ def test_list_of_events():
     _test(Events.ITERATION_STARTED(once=1) | Events.ITERATION_STARTED(once=1), [1, 1])
     _test(Events.ITERATION_STARTED(once=1) | Events.ITERATION_STARTED(once=10), [1, 10])
     _test(Events.ITERATION_STARTED(once=1) | Events.ITERATION_STARTED(every=3), [1, 3, 6, 9, 12, 15])
+    _test(Events.ITERATION_STARTED(once=8) | Events.ITERATION_STARTED(before=3), [1, 2, 8])
+    _test(Events.ITERATION_STARTED(once=1) | Events.ITERATION_STARTED(after=12), [1, 13, 14, 15])
